@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -36,7 +37,9 @@
 #include <sys/mman.h>
 #include <sys/param.h>
 
-int create_dirs(char *filename, mode_t mode)
+#include "textfile.h"
+
+int create_dirs(const char *filename, const mode_t mode)
 {
 	struct stat st;
 	char dir[PATH_MAX + 1], *prev, *next;
@@ -70,7 +73,7 @@ int create_dirs(char *filename, mode_t mode)
 	return 0;
 }
 
-int create_file(char *filename, mode_t mode)
+int create_file(const char *filename, const mode_t mode)
 {
 	int fd;
 
@@ -86,7 +89,27 @@ int create_file(char *filename, mode_t mode)
 	return 0;
 }
 
-static inline int write_key_value(int fd, char *key, char *value)
+int create_name(char *buf, size_t size, const char *path, const char *address, const char *name)
+{
+	return snprintf(buf, size, "%s/%s/%s", path, address, name);
+}
+
+static inline char *find_key(char *map, const char *key, size_t len, int icase)
+{
+	char *off = (icase) ? strcasestr(map, key) : strstr(map, key);
+
+	while (off && ((off > map && *(off - 1) != '\r' &&
+				*(off - 1) != '\n') || *(off + len) != ' ')) {
+		if (icase)
+			off = strcasestr(off + len, key);
+		else
+			off = strstr(off + len, key);
+	}
+
+	return off;
+}
+
+static inline int write_key_value(int fd, const char *key, const char *value)
 {
 	char *str;
 	size_t size;
@@ -108,18 +131,7 @@ static inline int write_key_value(int fd, char *key, char *value)
 	return err;
 }
 
-static inline char *find_key(char *map, char *key, size_t len)
-{
-	char *off = strstr(map, key);
-
-	while (off && ((off > map && *(off - 1) != '\r' &&
-				*(off - 1) != '\n') || *(off + len) != ' '))
-		off = strstr(off + len, key);
-
-	return off;
-}
-
-int textfile_put(char *pathname, char *key, char *value)
+static int write_key(const char *pathname, const char *key, const char *value, int icase)
 {
 	struct stat st;
 	char *map, *off, *end, *str;
@@ -143,22 +155,26 @@ int textfile_put(char *pathname, char *key, char *value)
 	size = st.st_size;
 
 	if (!size) {
-		pos = lseek(fd, size, SEEK_SET);
-		err = write_key_value(fd, key, value);
+		if (value) {
+			pos = lseek(fd, size, SEEK_SET);
+			err = write_key_value(fd, key, value);
+		}
 		goto unlock;
 	}
 
-	map = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_LOCKED, fd, 0);
+	map = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_LOCKED, fd, 0);
 	if (!map || map == MAP_FAILED) {
 		err = errno;
 		goto unlock;
 	}
 
-	off = find_key(map, key, strlen(key));
+	off = find_key(map, key, strlen(key), icase);
 	if (!off) {
-		munmap(map, size);
-		pos = lseek(fd, size, SEEK_SET);
-		err = write_key_value(fd, key, value);
+		if (value) {
+			munmap(map, size);
+			pos = lseek(fd, size, SEEK_SET);
+			err = write_key_value(fd, key, value);
+		}
 		goto unlock;
 	}
 
@@ -176,9 +192,14 @@ int textfile_put(char *pathname, char *key, char *value)
 	len = size - (end - map);
 	if (!len) {
 		munmap(map, size);
-		ftruncate(fd, base);
+		if (ftruncate(fd, base) < 0) {
+			err = errno;
+			goto unlock;
+		}
 		pos = lseek(fd, base, SEEK_SET);
-		err = write_key_value(fd, key, value);
+		if (value)
+			err = write_key_value(fd, key, value);
+
 		goto unlock;
 	}
 
@@ -196,11 +217,17 @@ int textfile_put(char *pathname, char *key, char *value)
 	memcpy(str, end, len);
 
 	munmap(map, size);
-	ftruncate(fd, base);
+	if (ftruncate(fd, base) < 0) {
+		err = errno;
+		free(str);
+		goto unlock;
+	}
 	pos = lseek(fd, base, SEEK_SET);
-	err = write_key_value(fd, key, value);
+	if (value)
+		err = write_key_value(fd, key, value);
 
-	write(fd, str, len);
+	if (write(fd, str, len) < 0)
+		err = errno;
 
 	free(str);
 
@@ -219,7 +246,7 @@ close:
 	return -err;
 }
 
-char *textfile_get(char *pathname, char *key)
+static char *read_key(const char *pathname, const char *key, int icase)
 {
 	struct stat st;
 	char *map, *off, *end, *str = NULL;
@@ -249,7 +276,7 @@ char *textfile_get(char *pathname, char *key)
 	}
 
 	len = strlen(key);
-	off = find_key(map, key, len);
+	off = find_key(map, key, len, icase);
 	if (!off) {
 		err = EILSEQ;
 		goto unmap;
@@ -282,3 +309,114 @@ close:
 
 	return str;
 }
+
+int textfile_put(const char *pathname, const char *key, const char *value)
+{
+	return write_key(pathname, key, value, 0);
+}
+
+int textfile_del(const char *pathname, const char *key)
+{
+	return write_key(pathname, key, NULL, 0);
+}
+
+char *textfile_get(const char *pathname, const char *key)
+{
+	return read_key(pathname, key, 0);
+}
+
+char *textfile_caseget(const char *pathname, const char *key)
+{
+	return read_key(pathname, key, 1);
+}
+
+int textfile_foreach(const char *pathname,
+		void (*func)(char *key, char *value, void *data), void *data)
+{
+	struct stat st;
+	char *map, *off, *end, *key, *value;
+	off_t size; size_t len;
+	int fd, err = 0;
+
+	fd = open(pathname, O_RDONLY);
+	if (fd < 0)
+		return -errno;
+
+	if (flock(fd, LOCK_SH) < 0) {
+		err = errno;
+		goto close;
+	}
+
+	if (fstat(fd, &st) < 0) {
+		err = errno;
+		goto unlock;
+	}
+
+	size = st.st_size;
+
+	map = mmap(NULL, size, PROT_READ, MAP_SHARED, fd, 0);
+	if (!map || map == MAP_FAILED) {
+		err = errno;
+		goto unlock;
+	}
+
+	off = map;
+
+	while (1) {
+		end = strpbrk(off, " ");
+		if (!end) {
+			err = EILSEQ;
+			break;
+		}
+
+		len = end - off;
+
+		key = malloc(len + 1);
+		if (!key) {
+			err = errno;
+			break;
+		}
+
+		memset(key, 0, len + 1);
+		memcpy(key, off, len);
+
+		off = end + 1;
+
+		end = strpbrk(off, "\r\n");
+		if (!end) {
+			err = EILSEQ;
+			free(key);
+			break;
+		}
+
+		len = end - off;
+
+		value = malloc(len + 1);
+		if (!value) {
+			err = errno;
+			free(key);
+			break;
+		}
+
+		memset(value, 0, len + 1);
+		memcpy(value, off, len);
+
+		func(key, value, data);
+
+		free(key);
+		free(value);
+
+		off = end + 1;
+	}
+
+	munmap(map, size);
+
+unlock:
+	flock(fd, LOCK_UN);
+
+close:
+	close(fd);
+	errno = err;
+
+	return 0;
+}
