@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,8 +21,20 @@
  *
  */
 
-int create_dirs(char *filename, mode_t mode);
-int create_file(char *filename, mode_t mode);
+#ifndef __TEXTFILE_H
+#define __TEXTFILE_H
 
-int textfile_put(char *pathname, char *key, char *value);
-char *textfile_get(char *pathname, char *key);
+int create_dirs(const char *filename, const mode_t mode);
+int create_file(const char *filename, const mode_t mode);
+int create_name(char *buf, size_t size, const char *path,
+				const char *address, const char *name);
+
+int textfile_put(const char *pathname, const char *key, const char *value);
+int textfile_del(const char *pathname, const char *key);
+char *textfile_get(const char *pathname, const char *key);
+char *textfile_caseget(const char *pathname, const char *key);
+
+int textfile_foreach(const char *pathname,
+		void (*func)(char *key, char *value, void *data), void *data);
+
+#endif /* __TEXTFILE_H */
