@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -56,12 +56,9 @@ struct kword cfg_keyword[] = {
 	{ "name",		K_NAME		},
 	{ "class",		K_CLASS		},
 	{ "voice",		K_VOICE		},
-	{ "inqmode",		K_INQMODE	},
 	{ "pageto",		K_PAGETO	},
-	{ "auth",		K_AUTH		},
-	{ "encrypt",		K_ENCRYPT	},
-	{ "pin_helper",		K_PINHELP	},
-	{ "dbus_pin_helper",	K_DBUSPINHELP	},
+	{ "discovto",		K_DISCOVTO	},
+	{ "passkey",		K_PASSKEY	},
 
 	{ "yes",		K_YES		},
 	{ "no",			K_NO		},
