@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,8 +34,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
-#include <syslog.h>
-#include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
@@ -47,7 +45,7 @@
 #include "glib-ectomy.h"
 
 #include "hcid.h"
-#include "lib.h"
+#include "sdp.h"
 
 struct hcid_opts hcid;
 struct device_opts default_device;
@@ -66,8 +64,9 @@ static void usage(void)
 static inline void init_device_defaults(struct device_opts *device_opts)
 {
 	memset(device_opts, 0, sizeof(*device_opts));
-	device_opts->scan = SCAN_PAGE | SCAN_INQUIRY;
+	device_opts->scan = SCAN_PAGE;
 	device_opts->name = strdup("BlueZ");
+	device_opts->discovto = HCID_DEFAULT_DISCOVERABLE_TIMEOUT;
 }
 
 struct device_opts *alloc_device_opts(char *ref)
@@ -76,12 +75,12 @@ struct device_opts *alloc_device_opts(char *ref)
 
 	device = malloc(sizeof(struct device_list));
 	if (!device) {
-		syslog(LOG_INFO, "Can't allocate devlist opts buffer: %s (%d)",
+		info("Can't allocate devlist opts buffer: %s (%d)",
 							strerror(errno), errno);
 		exit(1);
 	}
 
-	device->ref = ref;
+	device->ref = strdup(ref);
 	device->next = device_list;
 	device_list = device;
 
@@ -128,8 +127,7 @@ static struct device_opts *get_device_opts(int sock, int hdev)
 	struct hci_dev_info di;
 
 	/* First try to get BD_ADDR based settings ... */
-	di.dev_id = hdev;
-	if (!ioctl(sock, HCIGETDEVINFO, (void *) &di)) {
+	if (!hci_devinfo(hdev, &di) < 0) {
 		char addr[18];
 		ba2str(&di.bdaddr, addr);
 		device_opts = find_device_opts(addr);
@@ -149,11 +147,110 @@ static struct device_opts *get_device_opts(int sock, int hdev)
 	return device_opts;
 }
 
+int get_discoverable_timeout(int hdev)
+{
+	int sock, timeout;
+	struct device_opts *device_opts = NULL;
+	struct hci_dev_info di;
+	char addr[18];
+
+	if (hdev < 0)
+		return HCID_DEFAULT_DISCOVERABLE_TIMEOUT;
+
+	sock = hci_open_dev(hdev);
+	if (sock < 0)
+		goto no_address;
+
+	if (!hci_devinfo(hdev, &di) < 0) {
+		close(sock);
+		goto no_address;
+	}
+
+	close(sock);
+
+	if (read_discoverable_timeout(&di.bdaddr, &timeout) == 0)
+		return timeout;
+
+	ba2str(&di.bdaddr, addr);
+	device_opts = find_device_opts(addr);
+
+no_address:
+	if (!device_opts) {
+		char ref[8];
+		snprintf(ref, sizeof(ref) - 1, "hci%d", hdev);
+		device_opts = find_device_opts(ref);
+	}
+
+	if (!device_opts)
+		device_opts = &default_device;
+
+	return device_opts->discovto;
+}
+
+/* 
+ * Device name expansion 
+ *   %d - device id
+ */
+static char *expand_name(char *dst, int size, char *str, int dev_id)
+{
+	register int sp, np, olen;
+	char *opt, buf[10];
+
+	if (!str && !dst)
+		return NULL;
+
+	sp = np = 0;
+	while (np < size - 1 && str[sp]) {
+		switch (str[sp]) {
+		case '%':
+			opt = NULL;
+
+			switch (str[sp+1]) {
+			case 'd':
+				sprintf(buf, "%d", dev_id);
+				opt = buf;
+				break;
+
+			case 'h':
+				opt = hcid.host_name;
+				break;
+
+			case '%':
+				dst[np++] = str[sp++];
+				/* fall through */
+			default:
+				sp++;
+				continue;
+			}
+
+			if (opt) {
+				/* substitute */
+				olen = strlen(opt);
+				if (np + olen < size - 1)
+					memcpy(dst + np, opt, olen);
+				np += olen;
+			}
+			sp += 2;
+			continue;
+
+		case '\\':
+			sp++;
+			/* fall through */
+		default:
+			dst[np++] = str[sp++];
+			break;
+		}
+	}
+	dst[np] = '\0';
+	return dst;
+}
+
 static void configure_device(int hdev)
 {
 	struct device_opts *device_opts;
 	struct hci_dev_req dr;
 	struct hci_dev_info di;
+	char mode[14];
 	int s;
 
 	/* Do configuration in the separate process */
@@ -161,67 +258,68 @@ static void configure_device(int hdev)
 		case 0:
 			break;
 		case -1:
-			syslog(LOG_ERR, "Fork failed. Can't init device hci%d: %s (%d)",
+			error("Fork failed. Can't init device hci%d: %s (%d)",
 						hdev, strerror(errno), errno);
 		default:
 			return;
 	}
 
-	set_title("hci%d config", hdev);
-
 	if ((s = hci_open_dev(hdev)) < 0) {
-		syslog(LOG_ERR, "Can't open device hci%d: %s (%d)",
+		error("Can't open device hci%d: %s (%d)",
 						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
-	di.dev_id = hdev;
-	if (ioctl(s, HCIGETDEVINFO, (void *) &di) < 0)
+	if (hci_devinfo(hdev, &di) < 0)
 		exit(1);
 
 	if (hci_test_bit(HCI_RAW, &di.flags))
 		exit(0);
 
+	memset(&dr, 0, sizeof(dr));
+
 	dr.dev_id   = hdev;
 	device_opts = get_device_opts(s, hdev);
 
+	/* Set default discoverable timeout if not set */
+	if (!(device_opts->flags & (1 << HCID_SET_DISCOVTO)))
+		device_opts->discovto = HCID_DEFAULT_DISCOVERABLE_TIMEOUT;
+
 	/* Set scan mode */
+	if (!read_device_mode(&di.bdaddr, mode, sizeof(mode))) {
+		if (!strcmp(mode, MODE_OFF))
+			device_opts->scan = SCAN_DISABLED;
+		else if (!strcmp(mode, MODE_CONNECTABLE))
+			device_opts->scan = SCAN_PAGE;
+		else if (!strcmp(mode, MODE_DISCOVERABLE)) {
+			/* Set discoverable only if timeout is 0 */
+			if (!get_discoverable_timeout(hdev))
+				device_opts->scan = SCAN_PAGE | SCAN_INQUIRY;
+			else
+				device_opts->scan = SCAN_PAGE;
+		}
+	}
+
 	dr.dev_opt = device_opts->scan;
 	if (ioctl(s, HCISETSCAN, (unsigned long) &dr) < 0) {
-		syslog(LOG_ERR, "Can't set scan mode on hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
-	}
-
-	/* Set authentication */
-	if (device_opts->auth)
-		dr.dev_opt = AUTH_ENABLED;
-	else
-		dr.dev_opt = AUTH_DISABLED;
-
-	if (ioctl(s, HCISETAUTH, (unsigned long) &dr) < 0) {
-		syslog(LOG_ERR, "Can't set auth on hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
-	}
-
-	/* Set encryption */
-	if (device_opts->encrypt)
-		dr.dev_opt = ENCRYPT_P2P;
-	else
-		dr.dev_opt = ENCRYPT_DISABLED;
-
-	if (ioctl(s, HCISETENCRYPT, (unsigned long) &dr) < 0) {
-		syslog(LOG_ERR, "Can't set encrypt on hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
+		error("Can't set scan mode on hci%d: %s (%d)",
+				hdev, strerror(errno), errno);
 	}
 
 	/* Set device name */
 	if ((device_opts->flags & (1 << HCID_SET_NAME)) && device_opts->name) {
 		change_local_name_cp cp;
 		write_ext_inquiry_response_cp ip;
+		char name[249];
 		uint8_t len;
 
-		memset(cp.name, 0, sizeof(cp.name));
-		expand_name((char *) cp.name, sizeof(cp.name), device_opts->name, hdev);
+		memset(name, 0, sizeof(name));
+		if (read_local_name(&di.bdaddr, name) < 0) {
+			memset(cp.name, 0, sizeof(cp.name));
+			expand_name((char *) cp.name, sizeof(cp.name),
+						device_opts->name, hdev);
+		} else
+			memcpy(cp.name, name, sizeof(cp.name));
 
 		ip.fec = 0x00;
 		memset(ip.data, 0, sizeof(ip.data));
@@ -244,10 +342,16 @@ static void configure_device(int hdev)
 
 	/* Set device class */
 	if ((device_opts->flags & (1 << HCID_SET_CLASS))) {
-		uint32_t class = htobl(device_opts->class);
 		write_class_of_dev_cp cp;
+		uint32_t class;
+		uint8_t cls[3];
+
+		if (read_local_class(&di.bdaddr, cls) < 0) {
+			class = htobl(device_opts->class);
+			memcpy(cp.dev_class, &class, 3);
+		} else
+			memcpy(cp.dev_class, cls, 3);
 
-		memcpy(cp.dev_class, &class, 3);
 		hci_send_cmd(s, OGF_HOST_CTL, OCF_WRITE_CLASS_OF_DEV,
 					WRITE_CLASS_OF_DEV_CP_SIZE, &cp);
 	}
@@ -261,30 +365,6 @@ static void configure_device(int hdev)
 					WRITE_VOICE_SETTING_CP_SIZE, &cp);
 	}
 
-	/* Set inquiry mode */
-	if ((device_opts->flags & (1 << HCID_SET_INQMODE))) {
-		write_inquiry_mode_cp cp;
-
-		switch (device_opts->inqmode) {
-		case 2:
-			if (di.features[6] & LMP_EXT_INQ) {
-				cp.mode = 2;
-				break;
-			}
-		case 1:
-			if (di.features[3] & LMP_RSSI_INQ) {
-				cp.mode = 1;
-				break;
-			}
-		default:
-			cp.mode = 0;
-			break;
-		}
-
-		hci_send_cmd(s, OGF_HOST_CTL, OCF_WRITE_INQUIRY_MODE,
-					WRITE_INQUIRY_MODE_CP_SIZE, &cp);
-	}
-
 	/* Set page timeout */
 	if ((device_opts->flags & (1 << HCID_SET_PAGETO))) {
 		write_page_timeout_cp cp;
@@ -302,110 +382,121 @@ static void init_device(int hdev)
 	struct device_opts *device_opts;
 	struct hci_dev_req dr;
 	struct hci_dev_info di;
-	int s;
+	int dd, dev_id = hdev;
 
 	/* Do initialization in the separate process */
 	switch (fork()) {
 		case 0:
 			break;
 		case -1:
-			syslog(LOG_ERR, "Fork failed. Can't init device hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
+			error("Fork failed. Can't init device hci%d: %s (%d)",
+					dev_id, strerror(errno), errno);
 		default:
 			return;
 	}
 
-	set_title("hci%d init", hdev);
-
-	if ((s = hci_open_dev(hdev)) < 0) {
-		syslog(LOG_ERR, "Can't open device hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
+	dd = hci_open_dev(dev_id);
+	if (dd < 0) {
+		error("Can't open device hci%d: %s (%d)",
+					dev_id, strerror(errno), errno);
 		exit(1);
 	}
 
 	/* Start HCI device */
-	if (ioctl(s, HCIDEVUP, hdev) < 0 && errno != EALREADY) {
-		syslog(LOG_ERR, "Can't init device hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
+	if (ioctl(dd, HCIDEVUP, dev_id) < 0 && errno != EALREADY) {
+		error("Can't init device hci%d: %s (%d)",
+					dev_id, strerror(errno), errno);
 		exit(1);
 	}
 
-	di.dev_id = hdev;
-	if (ioctl(s, HCIGETDEVINFO, (void *) &di) < 0)
+	if (hci_devinfo(dev_id, &di) < 0)
 		exit(1);
 
 	if (hci_test_bit(HCI_RAW, &di.flags))
 		exit(0);
 
-	dr.dev_id   = hdev;
-	device_opts = get_device_opts(s, hdev);
+	memset(&dr, 0, sizeof(dr));
+	dr.dev_id   = dev_id;
+	device_opts = get_device_opts(dd, dev_id);
 
 	/* Set packet type */
 	if ((device_opts->flags & (1 << HCID_SET_PTYPE))) {
 		dr.dev_opt = device_opts->pkt_type;
-		if (ioctl(s, HCISETPTYPE, (unsigned long) &dr) < 0) {
-			syslog(LOG_ERR, "Can't set packet type on hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
+		if (ioctl(dd, HCISETPTYPE, (unsigned long) &dr) < 0) {
+			error("Can't set packet type on hci%d: %s (%d)",
+					dev_id, strerror(errno), errno);
 		}
 	}
 
 	/* Set link mode */
 	if ((device_opts->flags & (1 << HCID_SET_LM))) {
 		dr.dev_opt = device_opts->link_mode;
-		if (ioctl(s, HCISETLINKMODE, (unsigned long) &dr) < 0) {
-			syslog(LOG_ERR, "Can't set link mode on hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
+		if (ioctl(dd, HCISETLINKMODE, (unsigned long) &dr) < 0) {
+			error("Can't set link mode on hci%d: %s (%d)",
+					dev_id, strerror(errno), errno);
 		}
 	}
 
 	/* Set link policy */
 	if ((device_opts->flags & (1 << HCID_SET_LP))) {
 		dr.dev_opt = device_opts->link_policy;
-		if (ioctl(s, HCISETLINKPOL, (unsigned long) &dr) < 0) {
-			syslog(LOG_ERR, "Can't set link policy on hci%d: %s (%d)",
-						hdev, strerror(errno), errno);
+		if (ioctl(dd, HCISETLINKPOL, (unsigned long) &dr) < 0) {
+			error("Can't set link policy on hci%d: %s (%d)",
+					dev_id, strerror(errno), errno);
 		}
 	}
 
 	exit(0);
 }
 
+static void device_devreg_setup(int dev_id)
+{
+	if (hcid.auto_init)
+		init_device(dev_id);
+	hcid_dbus_register_device(dev_id);
+}
+
+static void device_devup_setup(int dev_id)
+{
+	add_device(dev_id);
+	if (hcid.auto_init)
+		configure_device(dev_id);
+	if (hcid.security)
+		start_security_manager(dev_id);
+	start_device(dev_id);
+	hcid_dbus_start_device(dev_id);
+}
+
 static void init_all_devices(int ctl)
 {
 	struct hci_dev_list_req *dl;
 	struct hci_dev_req *dr;
 	int i;
 
-	if (!(dl = malloc(HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t)))) {
-		syslog(LOG_INFO, "Can't allocate devlist buffer: %s (%d)",
+	dl = malloc(HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t));
+	if (!dl) {
+		info("Can't allocate devlist buffer: %s (%d)",
 							strerror(errno), errno);
 		exit(1);
 	}
+
+	memset(dl, 0, HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t));
 	dl->dev_num = HCI_MAX_DEV;
 	dr = dl->dev_req;
 
 	if (ioctl(ctl, HCIGETDEVLIST, (void *) dl) < 0) {
-		syslog(LOG_INFO, "Can't get device list: %s (%d)",
+		info("Can't get device list: %s (%d)",
 							strerror(errno), errno);
 		exit(1);
 	}
 
 	for (i = 0; i < dl->dev_num; i++, dr++) {
-		if (hcid.auto_init)
-			init_device(dr->dev_id);
-
-		if (hcid.auto_init && hci_test_bit(HCI_UP, &dr->dev_opt))
-			configure_device(dr->dev_id);
-
-		if (hcid.security && hci_test_bit(HCI_UP, &dr->dev_opt))
-			start_security_manager(dr->dev_id);
-
-#ifdef ENABLE_DBUS
-		hcid_dbus_register_device(dr->dev_id);
-
-		if (hci_test_bit(HCI_UP, &dr->dev_opt))
-			hcid_dbus_dev_up(dr->dev_id);
-#endif
+		info("HCI dev %d registered", dr->dev_id);
+		device_devreg_setup(dr->dev_id);
+		if (hci_test_bit(HCI_UP, &dr->dev_opt)) {
+			info("HCI dev %d already up", dr->dev_id);
+			device_devup_setup(dr->dev_id);
+		}
 	}
 
 	free(dl);
@@ -413,22 +504,12 @@ static void init_all_devices(int ctl)
 
 static void init_defaults(void)
 {
-	hcid.auto_init = 0;
-	hcid.security  = 0;
+	hcid.auto_init = 1;
+	hcid.security  = HCID_SEC_AUTO;
 
 	init_device_defaults(&default_device);
 }
 
-static void sig_usr1(int sig)
-{
-	toggle_pairing(0);
-}
-
-static void sig_usr2(int sig)
-{
-	toggle_pairing(1);
-}
-
 static void sig_term(int sig)
 {
 	g_main_quit(event_loop);
@@ -436,12 +517,14 @@ static void sig_term(int sig)
 
 static void sig_hup(int sig)
 {
-	syslog(LOG_INFO, "Reloading config file");
+	info("Reloading config file");
+
+	free_device_opts();
 
 	init_defaults();
 
 	if (read_config(hcid.config_file) < 0)
-		syslog(LOG_ERR, "Config reload failed");
+		error("Config reload failed");
 
 	init_security_data();
 
@@ -454,39 +537,27 @@ static inline void device_event(GIOChannel *chan, evt_stack_internal *si)
 
 	switch (sd->event) {
 	case HCI_DEV_REG:
-		syslog(LOG_INFO, "HCI dev %d registered", sd->dev_id);
-		if (hcid.auto_init)
-			init_device(sd->dev_id);
-#ifdef ENABLE_DBUS
-		hcid_dbus_register_device(sd->dev_id);
-#endif
+		info("HCI dev %d registered", sd->dev_id);
+		device_devreg_setup(sd->dev_id);
 		break;
 
 	case HCI_DEV_UNREG:
-		syslog(LOG_INFO, "HCI dev %d unregistered", sd->dev_id);
-#ifdef ENABLE_DBUS
+		info("HCI dev %d unregistered", sd->dev_id);
 		hcid_dbus_unregister_device(sd->dev_id);
-#endif
+		remove_device(sd->dev_id);
 		break;
 
 	case HCI_DEV_UP:
-		syslog(LOG_INFO, "HCI dev %d up", sd->dev_id);
-		if (hcid.auto_init)
-			configure_device(sd->dev_id);
-		if (hcid.security)
-			start_security_manager(sd->dev_id);
-#ifdef ENABLE_DBUS
-		hcid_dbus_dev_up(sd->dev_id);
-#endif
+		info("HCI dev %d up", sd->dev_id);
+		device_devup_setup(sd->dev_id);
 		break;
 
 	case HCI_DEV_DOWN:
-		syslog(LOG_INFO, "HCI dev %d down", sd->dev_id);
+		info("HCI dev %d down", sd->dev_id);
+		hcid_dbus_stop_device(sd->dev_id);
 		if (hcid.security)
 			stop_security_manager(sd->dev_id);
-#ifdef ENABLE_DBUS
-		hcid_dbus_dev_down(sd->dev_id);
-#endif
+		stop_device(sd->dev_id);
 		break;
 	}
 }
@@ -506,7 +577,7 @@ static gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer dat
 		if (err == G_IO_ERROR_AGAIN)
 			return TRUE;
 
-		syslog(LOG_ERR, "Read from control socket failed: %s (%d)",
+		error("Read from control socket failed: %s (%d)",
 							strerror(errno), errno);
 		g_main_quit(event_loop);
 		return FALSE;
@@ -533,35 +604,41 @@ static gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer dat
 	return TRUE;
 }
 
-extern int optind, opterr, optopt;
-extern char *optarg;
-
-int main(int argc, char *argv[], char *env[])
+int main(int argc, char *argv[])
 {
-	int daemon, dofork, opt, fd;
 	struct sockaddr_hci addr;
 	struct hci_filter flt;
 	struct sigaction sa;
 	GIOChannel *ctl_io;
-
-	daemon = 1; dofork = 1;
+	int opt, daemonize = 1, sdp = 0, experimental = 0;
 
 	/* Default HCId settings */
+	memset(&hcid, 0, sizeof(hcid));
+	hcid.auto_init   = 1;
 	hcid.config_file = HCID_CONFIG_FILE;
-	hcid.host_name   = get_host_name();
 	hcid.security    = HCID_SEC_AUTO;
 	hcid.pairing     = HCID_PAIRING_MULTI;
 
-	hcid.pin_file    = strdup(HCID_PIN_FILE);
-	hcid.pin_helper  = strdup(HCID_PIN_HELPER);
-	hcid.key_file    = strdup(HCID_KEY_FILE);
+	if (gethostname(hcid.host_name, sizeof(hcid.host_name) - 1) < 0)
+		strcpy(hcid.host_name, "noname");
+
+	strcpy((char *) hcid.pin_code, "BlueZ");
+	hcid.pin_len = 5;
 
 	init_defaults();
 
-	while ((opt = getopt(argc, argv, "f:n")) != EOF) {
+	while ((opt = getopt(argc, argv, "nsxf:")) != EOF) {
 		switch (opt) {
 		case 'n':
-			daemon = 0;
+			daemonize = 0;
+			break;
+
+		case 's':
+			sdp = 1;
+			break;
+
+		case 'x':
+			experimental = 1;
 			break;
 
 		case 'f':
@@ -574,28 +651,14 @@ int main(int argc, char *argv[], char *env[])
 		}
 	}
 
-	if (daemon) {
-		if (dofork && fork())
-			exit(0);
-
-		/* Direct stdin,stdout,stderr to '/dev/null' */
-		fd = open("/dev/null", O_RDWR);
-		dup2(fd, 0); dup2(fd, 1); dup2(fd, 2);
-		close(fd);
-
-		setsid();
-
-		chdir("/");
+	if (daemonize && daemon(0, 0)) {
+		error("Can't daemonize: %s (%d)", strerror(errno), errno);
+		exit(1);
 	}
 
 	umask(0077);
 
-	init_title(argc, argv, env, "hcid: ");
-	set_title("initializing");
-
-	/* Start logging to syslog and stderr */
-	openlog("hcid", LOG_PID | LOG_NDELAY | LOG_PERROR, LOG_DAEMON);
-	syslog(LOG_INFO, "Bluetooth HCI daemon");
+	start_logging("hcid", "Bluetooth HCI daemon");
 
 	memset(&sa, 0, sizeof(sa));
 	sa.sa_flags = SA_NOCLDSTOP;
@@ -604,18 +667,16 @@ int main(int argc, char *argv[], char *env[])
 	sigaction(SIGINT,  &sa, NULL);
 	sa.sa_handler = sig_hup;
 	sigaction(SIGHUP, &sa, NULL);
-	sa.sa_handler = sig_usr1;
-	sigaction(SIGUSR1, &sa, NULL);
-	sa.sa_handler = sig_usr2;
-	sigaction(SIGUSR2, &sa, NULL);
 
 	sa.sa_handler = SIG_IGN;
 	sigaction(SIGCHLD, &sa, NULL);
 	sigaction(SIGPIPE, &sa, NULL);
 
+	enable_debug();
+
 	/* Create and bind HCI socket */
 	if ((hcid.sock = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
-		syslog(LOG_ERR, "Can't open HCI socket: %s (%d)",
+		error("Can't open HCI socket: %s (%d)",
 							strerror(errno), errno);
 		exit(1);
 	}
@@ -625,7 +686,7 @@ int main(int argc, char *argv[], char *env[])
 	hci_filter_set_ptype(HCI_EVENT_PKT, &flt);
 	hci_filter_set_event(EVT_STACK_INTERNAL, &flt);
 	if (setsockopt(hcid.sock, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
-		syslog(LOG_ERR, "Can't set filter: %s (%d)",
+		error("Can't set filter: %s (%d)",
 							strerror(errno), errno);
 		exit(1);
 	}
@@ -633,48 +694,57 @@ int main(int argc, char *argv[], char *env[])
 	addr.hci_family = AF_BLUETOOTH;
 	addr.hci_dev = HCI_DEV_NONE;
 	if (bind(hcid.sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		syslog(LOG_ERR, "Can't bind HCI socket: %s (%d)",
+		error("Can't bind HCI socket: %s (%d)",
 							strerror(errno), errno);
 		exit(1);
 	}
 
 	if (read_config(hcid.config_file) < 0)
-		syslog(LOG_ERR, "Config load failed");
+		error("Config load failed");
 
-#ifdef ENABLE_DBUS
-	if (hcid_dbus_init() == FALSE && hcid.dbus_pin_helper) {
-		syslog(LOG_ERR, "Unable to get on D-BUS");
+	init_devices();
+
+	if (hcid_dbus_init() < 0) {
+		error("Unable to get on D-Bus");
 		exit(1);
 	}
-#else
-	if (hcid.dbus_pin_helper) {
-		syslog(LOG_ERR, "D-BUS not configured in this build of hcid");
-		exit(1);
-	}
-#endif
+
+	if (experimental)
+		hcid_dbus_set_experimental();
 
 	init_security_data();
 
 	/* Create event loop */
 	event_loop = g_main_new(FALSE);
 
+	ctl_io = g_io_channel_unix_new(hcid.sock);
+	g_io_channel_set_close_on_unref(ctl_io, TRUE);
+
+	g_io_add_watch(ctl_io, G_IO_IN, io_stack_event, NULL);
+
 	/* Initialize already connected devices */
 	init_all_devices(hcid.sock);
 
-	set_title("processing events");
-
-	ctl_io = g_io_channel_unix_new(hcid.sock);
-	g_io_add_watch(ctl_io, G_IO_IN, io_stack_event, NULL);
+	if (sdp)
+		start_sdp_server();
 
 	/* Start event processor */
 	g_main_run(event_loop);
 
+	if (sdp)
+		stop_sdp_server();
+
 	free_device_opts();
 
-#ifdef ENABLE_DBUS
 	hcid_dbus_exit();
-#endif
 
-	syslog(LOG_INFO, "Exit.");
+	g_main_unref(event_loop);
+
+	g_io_channel_unref(ctl_io);
+
+	info("Exit");
+
+	stop_logging();
+
 	return 0;
 }
