@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,8 +36,9 @@
 #include <malloc.h>
 #include <string.h>
 #include <signal.h>
-#include <syslog.h>
 #include <time.h>
+#include <sys/time.h>
+#include <sys/param.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 
@@ -48,20 +49,149 @@
 #include "glib-ectomy.h"
 
 #include "hcid.h"
-#include "lib.h"
+#include "textfile.h"
+#include "list.h"
 
-static GIOChannel *io_chan[HCI_MAX_DEV];
+struct g_io_info {
+	GIOChannel	*channel;
+	int		watch_id;
+	int		pin_length;
+};
 
-static int pairing;
+static struct g_io_info io_data[HCI_MAX_DEV];
 
-void toggle_pairing(int enable)
+static int pairing = HCID_PAIRING_MULTI;
+
+static struct slist *hci_req_queue = NULL;
+
+struct hci_req_data *hci_req_data_new(int dev_id, const bdaddr_t *dba, uint16_t ogf, uint16_t ocf, int event, const void *cparam, int clen)
 {
-	if (enable)
-		pairing = hcid.pairing;
-	else
-		pairing = 0;
+	struct hci_req_data *data;
 
-	syslog(LOG_INFO, "Pairing %s", pairing ? "enabled" : "disabled");
+	data = malloc(sizeof(*data));
+	if (!data)
+		return NULL;
+
+	memset(data, 0, sizeof(*data));
+
+	data->cparam = malloc(clen);
+	if (!data->cparam) {
+		free(data);
+		return NULL;
+	}
+
+	memcpy(data->cparam, cparam, clen);
+
+	bacpy(&data->dba, dba);
+
+	data->dev_id = dev_id;
+	data->status = REQ_PENDING;
+	data->ogf    = ogf;
+	data->ocf    = ocf;
+	data->event  = event;
+	data->clen   = clen;
+
+	return data;
+}
+
+static int hci_req_find_by_devid(const void *data, const void *user_data)
+{
+	const struct hci_req_data *req = data;
+	const int *dev_id = user_data;
+
+	return (*dev_id - req->dev_id);
+}
+
+static void hci_req_queue_process(int dev_id)
+{
+	int dd, ret_val;
+
+	/* send the next pending cmd */
+	dd = hci_open_dev(dev_id);
+	do {
+		struct hci_req_data *data;
+		struct slist *l = slist_find(hci_req_queue, &dev_id, hci_req_find_by_devid);
+
+		if (!l)
+			break;
+
+		data = l->data;
+		data->status = REQ_SENT;
+		
+		ret_val = hci_send_cmd(dd, data->ogf, data->ocf, data->clen, data->cparam);
+		if (ret_val < 0) {
+			hci_req_queue = slist_remove(hci_req_queue, data);
+			free(data->cparam);
+			free(data);
+		}
+
+	} while(ret_val < 0);
+
+	hci_close_dev(dd);
+}
+
+void hci_req_queue_append(struct hci_req_data *data)
+{
+	struct slist *l;
+	struct hci_req_data *match;
+
+
+	hci_req_queue = slist_append(hci_req_queue, data);
+
+	l = slist_find(hci_req_queue, &data->dev_id, hci_req_find_by_devid);
+	match = l->data;
+
+	if (match->status == REQ_SENT)
+		return;
+
+	hci_req_queue_process(data->dev_id);
+}
+
+void hci_req_queue_remove(int dev_id, bdaddr_t *dba)
+{
+	struct slist *cur, *next;
+	struct hci_req_data *req;
+
+	for (cur = hci_req_queue; cur != NULL; cur = next) {
+		req = cur->data;
+		next = cur->next;
+		if ((req->dev_id != dev_id) || (bacmp(&req->dba, dba)))
+			continue;
+
+		hci_req_queue = slist_remove(hci_req_queue, req);
+		free(req->cparam);
+		free(req);
+	}
+}
+
+static void check_pending_hci_req(int dev_id, int event)
+{
+	struct hci_req_data *data;
+	struct slist *l;
+
+	if (!hci_req_queue)
+		return;
+
+	/* find the first element(pending)*/
+	l = slist_find(hci_req_queue, &dev_id, hci_req_find_by_devid);
+
+	if (!l)
+		return;
+
+	data = l->data;
+
+	/* skip if there is pending confirmation */
+	if (data->status == REQ_SENT) {
+		if (data->event != event)
+			return;
+
+		/* remove the confirmed cmd */
+		hci_req_queue = slist_remove(hci_req_queue, data);
+		free(data->cparam);
+		free(data);
+	}
+
+	hci_req_queue_process(dev_id);
 }
 
 static inline int get_bdaddr(int dev, bdaddr_t *sba, uint16_t handle, bdaddr_t *dba)
@@ -75,6 +205,8 @@ static inline int get_bdaddr(int dev, bdaddr_t *sba, uint16_t handle, bdaddr_t *
 	if (!cl)
 		return -ENOMEM;
 
+	memset(cl, 0, 10 * sizeof(*ci) + sizeof(*cl));
+
 	ba2str(sba, addr);
 	cl->dev_id = hci_devid(addr);
 	cl->conn_num = 10;
@@ -93,48 +225,34 @@ static inline int get_bdaddr(int dev, bdaddr_t *sba, uint16_t handle, bdaddr_t *
 		}
 
 	free(cl);
+
 	return -ENOENT;
 }
 
+static inline void update_lastseen(bdaddr_t *sba, bdaddr_t *dba)
+{
+	time_t t;
+	struct tm *tm;
+
+	t = time(NULL);
+	tm = gmtime(&t);
+
+	write_lastseen_info(sba, dba, tm);
+}
+
+static inline void update_lastused(bdaddr_t *sba, bdaddr_t *dba)
+{
+	time_t t;
+	struct tm *tm;
+
+	t = time(NULL);
+	tm = gmtime(&t);
+
+	write_lastused_info(sba, dba, tm);
+}
+
 /* Link Key handling */
 
-/* This function is not reentrable */
-static struct link_key *__get_link_key(int f, bdaddr_t *sba, bdaddr_t *dba)
-{
-	static struct link_key k;
-	struct link_key *key = NULL;
-	int r;
-
-	while ((r = read_n(f, &k, sizeof(k)))) {
-		if (r < 0) {
-			syslog(LOG_ERR, "Link key database read failed: %s (%d)",
-							strerror(errno), errno);
-			break;
-		}
-
-		if (!bacmp(&k.sba, sba) && !bacmp(&k.dba, dba)) {
-			key = &k;
-			break;
-		}
-	}
-	return key;
-}
-
-static struct link_key *get_link_key(bdaddr_t *sba, bdaddr_t *dba)
-{
-	struct link_key *key = NULL;
-	int f;
-
-	f = open(hcid.key_file, O_RDONLY);
-	if (f >= 0)
-		key = __get_link_key(f, sba, dba);
-	else if (errno != ENOENT)
-		syslog(LOG_ERR, "Link key database open failed: %s (%d)",
-							strerror(errno), errno);
-	close(f);
-	return key;
-}
-
 static void link_key_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 {
 	unsigned char key[16];
@@ -142,18 +260,9 @@ static void link_key_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 	int err;
 
 	ba2str(sba, sa); ba2str(dba, da);
-	syslog(LOG_INFO, "link_key_request (sba=%s, dba=%s)", sa, da);
+	info("link_key_request (sba=%s, dba=%s)", sa, da);
 
 	err = read_link_key(sba, dba, key);
-	if (err < 0) {
-		struct link_key *linkkey = get_link_key(sba, dba);
-		if (linkkey) {
-			memcpy(key, linkkey->key, 16);
-			linkkey->time = time(0);
-			err = 0;
-		}
-	}
-
 	if (err < 0) {
 		/* Link key not found */
 		hci_send_cmd(dev, OGF_LINK_CTL, OCF_LINK_KEY_NEG_REPLY, 6, dba);
@@ -167,71 +276,24 @@ static void link_key_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 	}
 }
 
-#if 0
-static void save_link_key(struct link_key *key)
-{
-	struct link_key *exist;
-	char sa[18], da[18];
-	int f, err;
-
-	f = open(hcid.key_file, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
-	if (f < 0) {
-		syslog(LOG_ERR, "Link key database open failed: %s (%d)",
-							strerror(errno), errno);
-		return;
-	}
-
-	/* Check if key already exist */
-	exist = __get_link_key(f, &key->sba, &key->dba);
-
-	err = 0;
-
-	if (exist) {
-		off_t o = lseek(f, 0, SEEK_CUR);
-		err = lseek(f, o - sizeof(*key), SEEK_SET);
-	} else
-		err = fcntl(f, F_SETFL, O_APPEND);
-
-	if (err < 0) {
-		syslog(LOG_ERR, "Link key database seek failed: %s (%d)",
-							strerror(errno), errno);
-		goto failed;
-	}
-	
-	if (write_n(f, key, sizeof(*key)) < 0) {
-		syslog(LOG_ERR, "Link key database write failed: %s (%d)",
-							strerror(errno), errno);
-	}
-
-	ba2str(&key->sba, sa); ba2str(&key->dba, da);
-	syslog(LOG_INFO, "%s link key %s %s", exist ? "Replacing" : "Saving", sa, da);
-
-failed:
-	close(f);
-}
-#endif
-
 static void link_key_notify(int dev, bdaddr_t *sba, void *ptr)
 {
 	evt_link_key_notify *evt = ptr;
 	bdaddr_t *dba = &evt->bdaddr;
-	struct link_key key;
 	char sa[18], da[18];
+	int dev_id;
 
 	ba2str(sba, sa); ba2str(dba, da);
-	syslog(LOG_INFO, "link_key_notify (sba=%s, dba=%s)", sa, da);
+	info("link_key_notify (sba=%s, dba=%s)", sa, da);
 
-	memcpy(key.key, evt->link_key, 16);
-	bacpy(&key.sba, sba);
-	bacpy(&key.dba, dba);
-	key.type = evt->key_type;
-	key.time = time(0);
+	dev_id = hci_devid(sa);
 
-#if 0
-	save_link_key(&key);
-#endif
+	write_link_key(sba, dba, evt->link_key, evt->key_type,
+						io_data[dev_id].pin_length);
 
-	write_link_key(sba, dba, evt->link_key, evt->key_type);
+	hcid_dbus_bonding_process_complete(sba, dba, 0);
+
+	io_data[dev_id].pin_length = -1;
 }
 
 static void return_link_keys(int dev, bdaddr_t *sba, void *ptr)
@@ -250,7 +312,7 @@ static void return_link_keys(int dev, bdaddr_t *sba, void *ptr)
 		bacpy(&dba, ptr); ba2str(&dba, da);
 		memcpy(key, ptr + 6, 16);
 
-		syslog(LOG_INFO, "return_link_keys (sba=%s, dba=%s)", sa, da);
+		info("return_link_keys (sba=%s, dba=%s)", sa, da);
 
 		ptr += 22;
 	}
@@ -258,151 +320,15 @@ static void return_link_keys(int dev, bdaddr_t *sba, void *ptr)
 
 /* PIN code handling */
 
-static int read_default_pin_code(void)
+void set_pin_length(bdaddr_t *sba, int length)
 {
-	char buf[17];
-	FILE *f; 
-	int len;
+	char addr[18];
+	int dev_id;
 
-	if (!(f = fopen(hcid.pin_file, "r"))) {
-		syslog(LOG_ERR, "Can't open PIN file %s: %s (%d)",
-					hcid.pin_file, strerror(errno), errno);
-		return -1;
-	}
+	ba2str(sba, addr);
+	dev_id = hci_devid(addr);
 
-	if (fgets(buf, sizeof(buf), f)) {
-		strtok(buf, "\n\r");
-		len = strlen(buf); 
-		memcpy(hcid.pin_code, buf, len);
-		hcid.pin_len = len;
-	} else {
-		syslog(LOG_ERR, "Can't read PIN file %s: %s (%d)",
-					hcid.pin_file, strerror(errno), errno);
-		len = -1;
-	}
-	fclose(f);
-	return len;
-}
-
-/*
-  PIN helper is an external app that asks user for a PIN. It can 
-  implement its own PIN  code generation policy and methods like
-  PIN look up in some database, etc. 
-  HCId expects following output from PIN helper:
-	PIN:12345678	-	PIN code
-	ERR		-	No PIN available
-*/
-
-static void call_pin_helper(int dev, bdaddr_t *sba, struct hci_conn_info *ci)
-{
-	pin_code_reply_cp pr;
-	struct sigaction sa;
-	char addr[18], str[512], *pin, name[249], tmp[497], *ptr;
-	FILE *pipe;
-	int i, ret, len;
-
-	/* Run PIN helper in the separate process */
-	switch (fork()) {
-		case 0:
-			break;
-		case -1:
-			syslog(LOG_ERR, "Can't fork PIN helper: %s (%d)",
-							strerror(errno), errno);
-		default:
-			return;
-	}
-
-	if (access(hcid.pin_helper, R_OK | X_OK)) {
-		syslog(LOG_ERR, "Can't exec PIN helper %s: %s (%d)",
-					hcid.pin_helper, strerror(errno), errno);
-		goto reject;
-	}
-
-	memset(name, 0, sizeof(name));
-	read_device_name(sba, &ci->bdaddr, name);
-	//hci_remote_name(dev, &ci->bdaddr, sizeof(name), name, 0);
-
-	memset(tmp, 0, sizeof(tmp));
-	ptr = tmp;
-
-	for (i = 0; i < 248 && name[i]; i++)
-		if (isprint(name[i])) {
-			switch (name[i]) {
-			case '"':
-			case '`':
-			case '$':
-			case '|':
-			case '>':
-			case '<':
-			case '&':
-			case ';':
-			case '\\':
-				*ptr++ = '\\';
-			}
-			*ptr++ = name[i];
-		} else {
-			name[i] = '.';
-			*ptr++ = '.';
-		}
-
-	ba2str(&ci->bdaddr, addr);
-	snprintf(str, sizeof(str), "%s %s %s \"%s\"", hcid.pin_helper,
-					ci->out ? "out" : "in", addr, tmp);
-
-	setenv("PATH", "/bin:/usr/bin:/usr/local/bin", 1);
-
-	memset(&sa, 0, sizeof(sa));
-	sa.sa_flags = SA_NOCLDSTOP;
-	sa.sa_handler = SIG_DFL;
-	sigaction(SIGCHLD, &sa, NULL);
-
-	pipe = popen(str, "r");
-	if (!pipe) {
-		syslog(LOG_ERR, "Can't exec PIN helper: %s (%d)",
-							strerror(errno), errno);
-		goto reject;
-	}
-
-	pin = fgets(str, sizeof(str), pipe);
-	ret = pclose(pipe);
-
-	if (!pin || strlen(pin) < 5)
-		goto nopin;
-
-	strtok(pin, "\n\r");
-
-	if (strncmp("PIN:", pin, 4))
-		goto nopin;
-
-	pin += 4;
-	len  = strlen(pin);
-
-	memset(&pr, 0, sizeof(pr));
-	bacpy(&pr.bdaddr, &ci->bdaddr);
-	memcpy(pr.pin_code, pin, len);
-	pr.pin_len = len;
-	hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_REPLY,
-			PIN_CODE_REPLY_CP_SIZE, &pr);
-	exit(0);
-
-nopin:
-	if (!pin || strncmp("ERR", pin, 3))
-		syslog(LOG_ERR, "PIN helper exited abnormally with code %d", ret);
-
-reject:
-	hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_NEG_REPLY, 6, &ci->bdaddr);
-	exit(0);
-}
-
-static void request_pin(int dev, bdaddr_t *sba, struct hci_conn_info *ci)
-{
-#ifdef ENABLE_DBUS
-	if (hcid.dbus_pin_helper) {
-		hcid_dbus_request_pin(dev, ci);
-		return;
-	}
-#endif
-	call_pin_helper(dev, sba, ci);
+	io_data[dev_id].pin_length = length;
 }
 
 static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
@@ -410,24 +336,26 @@ static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 	pin_code_reply_cp pr;
 	struct hci_conn_info_req *cr;
 	struct hci_conn_info *ci;
+	unsigned char key[16];
 	char sa[18], da[18], pin[17];
-	int pinlen;
+	int err, pinlen;
 
 	memset(&pr, 0, sizeof(pr));
 	bacpy(&pr.bdaddr, dba);
 
 	ba2str(sba, sa); ba2str(dba, da);
-	syslog(LOG_INFO, "pin_code_request (sba=%s, dba=%s)", sa, da);
+	info("pin_code_request (sba=%s, dba=%s)", sa, da);
 
 	cr = malloc(sizeof(*cr) + sizeof(*ci));
 	if (!cr)
 		return;
 
+	memset(cr, 0, sizeof(*cr) + sizeof(*ci));
+
 	bacpy(&cr->bdaddr, dba);
 	cr->type = ACL_LINK;
 	if (ioctl(dev, HCIGETCONNINFO, (unsigned long) cr) < 0) {
-		syslog(LOG_ERR, "Can't get conn info: %s (%d)",
-							strerror(errno), errno);
+		error("Can't get conn info: %s (%d)", strerror(errno), errno);
 		goto reject;
 	}
 	ci = cr->conn_info;
@@ -436,10 +364,10 @@ static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 	pinlen = read_pin_code(sba, dba, pin);
 
 	if (pairing == HCID_PAIRING_ONCE) {
-		struct link_key *key = get_link_key(sba, dba);
-		if (key) {
+		err = read_link_key(sba, dba, key);
+		if (!err) {
 			ba2str(dba, da);
-			syslog(LOG_WARNING, "PIN code request for already paired device %s", da);
+			error("PIN code request for already paired device %s", da);
 			goto reject;
 		}
 	} else if (pairing == HCID_PAIRING_NONE)
@@ -448,6 +376,7 @@ static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 	if (hcid.security == HCID_SEC_AUTO) {
 		if (!ci->out) {
 			/* Incomming connection */
+			set_pin_length(sba, hcid.pin_len);
 			memcpy(pr.pin_code, hcid.pin_code, hcid.pin_len);
 			pr.pin_len = hcid.pin_len;
 			hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_REPLY,
@@ -455,43 +384,62 @@ static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 		} else {
 			/* Outgoing connection */
 			if (pinlen > 0) {
+				set_pin_length(sba, pinlen);
 				memcpy(pr.pin_code, pin, pinlen);
 				pr.pin_len = pinlen;
 				hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_REPLY,
 					PIN_CODE_REPLY_CP_SIZE, &pr);
 			} else {
 				/* Let PIN helper handle that */ 
-				request_pin(dev, sba, ci);
+				hcid_dbus_request_pin(dev, sba, ci);
 			}
 		}
 	} else {
 		/* Let PIN helper handle that */ 
-		request_pin(dev, sba, ci);
+		hcid_dbus_request_pin(dev, sba, ci);
 	}
+
+	hcid_dbus_pending_pin_req_add(sba, &ci->bdaddr);
+
 	free(cr);
+
 	return;
 
 reject:
-	hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_NEG_REPLY, 6, dba);
 	free(cr);
+
+	hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_NEG_REPLY, 6, dba);
+
 	return;
 }
 
 static inline void cmd_status(int dev, bdaddr_t *sba, void *ptr)
 {
 	evt_cmd_status *evt = ptr;
+	uint16_t opcode = btohs(evt->opcode);
 
 	if (evt->status)
 		return;
 
-	if (evt->opcode == cmd_opcode_pack(OGF_LINK_CTL, OCF_INQUIRY))
+	if (opcode == cmd_opcode_pack(OGF_LINK_CTL, OCF_INQUIRY))
 		hcid_dbus_inquiry_start(sba);
 }
 
 static inline void cmd_complete(int dev, bdaddr_t *sba, void *ptr)
 {
 	evt_cmd_complete *evt = ptr;
-	switch (evt->opcode) {
+	uint16_t opcode = btohs(evt->opcode);
+	uint8_t status;
+
+	switch (opcode) {
+	case cmd_opcode_pack(OGF_LINK_CTL, OCF_PERIODIC_INQUIRY):
+		status = *((uint8_t *) ptr + EVT_CMD_COMPLETE_SIZE);
+		hcid_dbus_periodic_inquiry_start(sba, status);
+		break;
+	case cmd_opcode_pack(OGF_LINK_CTL, OCF_EXIT_PERIODIC_INQUIRY):
+		status = *((uint8_t *) ptr + EVT_CMD_COMPLETE_SIZE);
+		hcid_dbus_periodic_inquiry_exit(sba, status);
+		break;
 	case cmd_opcode_pack(OGF_LINK_CTL, OCF_INQUIRY_CANCEL):
 		hcid_dbus_inquiry_complete(sba);
 		break;
@@ -500,6 +448,11 @@ static inline void cmd_complete(int dev, bdaddr_t *sba, void *ptr)
 		break;
 	case cmd_opcode_pack(OGF_HOST_CTL, OCF_WRITE_SCAN_ENABLE):
 		hcid_dbus_setscan_enable_complete(sba);
+		break;
+	case cmd_opcode_pack(OGF_LINK_CTL, OCF_PIN_CODE_REPLY):
+	case cmd_opcode_pack(OGF_LINK_CTL, OCF_PIN_CODE_NEG_REPLY):
+		hcid_dbus_pin_code_reply(sba, ptr);
+		break;
 	};
 }
 
@@ -507,17 +460,22 @@ static inline void remote_name_information(int dev, bdaddr_t *sba, void *ptr)
 {
 	evt_remote_name_req_complete *evt = ptr;
 	bdaddr_t dba;
+	char name[249];
 
+	memset(name, 0, sizeof(name));
 	bacpy(&dba, &evt->bdaddr);
 
 	if (!evt->status) {
-		char name[249];
-		memset(name, 0, sizeof(name));
+		char *end;
 		memcpy(name, evt->name, 248);
+		/* It's ok to cast end between const and non-const since
+		 * we know it points to inside of name which is non-const */
+		if (!g_utf8_validate(name, -1, (const char **) &end))
+			*end = '\0';
 		write_device_name(sba, &dba, name);
-		hcid_dbus_remote_name(sba, &dba, name);
-	} else
-		hcid_dbus_remote_name_failed(sba, &dba, evt->status);
+	}
+
+	hcid_dbus_remote_name(sba, &dba, evt->status, name);
 }
 
 static inline void remote_version_information(int dev, bdaddr_t *sba, void *ptr)
@@ -551,7 +509,9 @@ static inline void inquiry_result(int dev, bdaddr_t *sba, int plen, void *ptr)
 			| (info->dev_class[1] << 8)
 			| (info->dev_class[2] << 16);
 
-		hcid_dbus_inquiry_result(sba, &info->bdaddr, class, 0);
+		hcid_dbus_inquiry_result(sba, &info->bdaddr, class, 0, NULL);
+
+		update_lastseen(sba, &info->bdaddr);
 
 		ptr += INQUIRY_INFO_SIZE;
 	}
@@ -573,7 +533,9 @@ static inline void inquiry_result_with_rssi(int dev, bdaddr_t *sba, int plen, vo
 				| (info->dev_class[2] << 16);
 
 			hcid_dbus_inquiry_result(sba, &info->bdaddr,
-							class, info->rssi);
+						class, info->rssi, NULL);
+
+			update_lastseen(sba, &info->bdaddr);
 
 			ptr += INQUIRY_INFO_WITH_RSSI_AND_PSCAN_MODE_SIZE;
 		}
@@ -585,7 +547,9 @@ static inline void inquiry_result_with_rssi(int dev, bdaddr_t *sba, int plen, vo
 				| (info->dev_class[2] << 16);
 
 			hcid_dbus_inquiry_result(sba, &info->bdaddr,
-							class, info->rssi);
+						class, info->rssi, NULL);
+
+			update_lastseen(sba, &info->bdaddr);
 
 			ptr += INQUIRY_INFO_WITH_RSSI_SIZE;
 		}
@@ -603,7 +567,10 @@ static inline void extended_inquiry_result(int dev, bdaddr_t *sba, int plen, voi
 			| (info->dev_class[1] << 8)
 			| (info->dev_class[2] << 16);
 
-		hcid_dbus_inquiry_result(sba, &info->bdaddr, class, info->rssi);
+		hcid_dbus_inquiry_result(sba, &info->bdaddr, class,
+						info->rssi, info->data);
+
+		update_lastseen(sba, &info->bdaddr);
 
 		ptr += EXTENDED_INQUIRY_INFO_SIZE;
 	}
@@ -623,27 +590,62 @@ static inline void remote_features_information(int dev, bdaddr_t *sba, void *ptr
 	write_features_info(sba, &dba, evt->features);
 }
 
-static inline void conn_complete(int dev, bdaddr_t *sba, void *ptr)
+static inline void conn_complete(int dev, int dev_id, bdaddr_t *sba, void *ptr)
 {
 	evt_conn_complete *evt = ptr;
+	char filename[PATH_MAX];
+	remote_name_req_cp cp_name;
+	bdaddr_t tmp;
+	struct hci_req_data *data;
+	char *str, *local_addr, *peer_addr;
+
+	hcid_dbus_conn_complete(sba, evt->status, evt->handle, &evt->bdaddr);
 
 	if (evt->status)
 		return;
 
-	hcid_dbus_conn_complete(sba, &evt->bdaddr);
+	update_lastused(sba, &evt->bdaddr);
+
+	/* Request remote name */
+	memset(&cp_name, 0, sizeof(cp_name));
+	bacpy(&cp_name.bdaddr, &evt->bdaddr);
+	cp_name.pscan_rep_mode = 0x02;
+
+	data = hci_req_data_new(dev_id, &evt->bdaddr, OGF_LINK_CTL,
+				OCF_REMOTE_NAME_REQ, EVT_REMOTE_NAME_REQ_COMPLETE,
+				&cp_name, REMOTE_NAME_REQ_CP_SIZE);
+
+	hci_req_queue_append(data);
+
+	/* check if the remote version needs be requested */
+	baswap(&tmp, sba); local_addr = batostr(&tmp);
+	baswap(&tmp, &evt->bdaddr); peer_addr = batostr(&tmp);
+
+	create_name(filename, sizeof(filename), STORAGEDIR, local_addr, "manufacturers");
+
+	str = textfile_get(filename, peer_addr);
+	if (!str) {
+		read_remote_version_cp cp;
+
+		cp.handle = evt->handle;
+
+		data = hci_req_data_new(dev_id, &evt->bdaddr, OGF_LINK_CTL,
+					OCF_READ_REMOTE_VERSION, EVT_READ_REMOTE_VERSION_COMPLETE,
+					&cp, READ_REMOTE_VERSION_CP_SIZE);
+
+		hci_req_queue_append(data);
+	} else
+		free(str);
+
+	free(local_addr);
+	free(peer_addr);
 }
 
 static inline void disconn_complete(int dev, bdaddr_t *sba, void *ptr)
 {
 	evt_disconn_complete *evt = ptr;
-	bdaddr_t dba;
 
-	if (evt->status)
-		return;
-
-	bacpy(&dba, BDADDR_ANY);
-
-	hcid_dbus_disconn_complete(sba, &dba, evt->reason);
+	hcid_dbus_disconn_complete(sba, evt->status, evt->handle, evt->reason);
 }
 
 static inline void auth_complete(int dev, bdaddr_t *sba, void *ptr)
@@ -651,33 +653,42 @@ static inline void auth_complete(int dev, bdaddr_t *sba, void *ptr)
 	evt_auth_complete *evt = ptr;
 	bdaddr_t dba;
 
-	if (get_bdaddr(dev, sba, evt->handle, &dba) < 0) 
+	if (get_bdaddr(dev, sba, btohs(evt->handle), &dba) < 0) 
 		return;
 
-	hcid_dbus_auth_complete(sba, &dba, evt->status);
+	if (evt->status)
+		hcid_dbus_bonding_process_complete(sba, &dba, evt->status);
 }
 
+static inline void conn_request(int dev, bdaddr_t *sba, void *ptr)
+{
+	evt_conn_request *evt = ptr;
+	uint32_t class = evt->dev_class[0] | (evt->dev_class[1] << 8)
+				| (evt->dev_class[2] << 16);
+
+	hcid_dbus_remote_class(sba, &evt->bdaddr, class);
+
+	write_remote_class(sba, &evt->bdaddr, class);
+}
 
 static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer data)
 {
 	unsigned char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
-	struct hci_dev_info *di = (void *) data;
+	struct hci_dev_info *di = data;
 	int type, dev;
 	size_t len;
 	hci_event_hdr *eh;
 	GIOError err;
 
 	if (cond & (G_IO_NVAL | G_IO_HUP | G_IO_ERR)) {
-		g_io_channel_close(chan);
-		free(data);
+		g_io_channel_unref(chan);
 		return FALSE;
 	}
 
 	if ((err = g_io_channel_read(chan, (gchar *) buf, sizeof(buf), &len))) {
 		if (err == G_IO_ERROR_AGAIN)
 			return TRUE;
-		g_io_channel_close(chan);
-		free(data);
+		g_io_channel_unref(chan);
 		return FALSE;
 	}
 
@@ -734,17 +745,25 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 		break;
 
 	case EVT_CONN_COMPLETE:
-		conn_complete(dev, &di->bdaddr, ptr);
+		conn_complete(dev, di->dev_id, &di->bdaddr, ptr);
 		break;
 
 	case EVT_DISCONN_COMPLETE:
 		disconn_complete(dev, &di->bdaddr, ptr);
 		break;
+
 	case EVT_AUTH_COMPLETE:
 		auth_complete(dev, &di->bdaddr, ptr);
 		break;
+
+	case EVT_CONN_REQUEST:
+		conn_request(dev, &di->bdaddr, ptr);
+		break;
 	}
 
+	/* Check for pending command request */
+	check_pending_hci_req(di->dev_id, eh->evt);
+
 	if (hci_test_bit(HCI_SECMGR, &di->flags))
 		return TRUE;
 
@@ -771,7 +790,7 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 
 void start_security_manager(int hdev)
 {
-	GIOChannel *chan = io_chan[hdev];
+	GIOChannel *chan = io_data[hdev].channel;
 	struct hci_dev_info *di;
 	struct hci_filter flt;
 	read_stored_link_key_cp cp;
@@ -780,10 +799,10 @@ void start_security_manager(int hdev)
 	if (chan)
 		return;
 
-	syslog(LOG_INFO, "Starting security manager %d", hdev);
+	info("Starting security manager %d", hdev);
 
 	if ((dev = hci_open_dev(hdev)) < 0) {
-		syslog(LOG_ERR, "Can't open device hci%d: %s (%d)",
+		error("Can't open device hci%d: %s (%d)",
 						hdev, strerror(errno), errno);
 		return;
 	}
@@ -807,8 +826,9 @@ void start_security_manager(int hdev)
 	hci_filter_set_event(EVT_CONN_COMPLETE, &flt);
 	hci_filter_set_event(EVT_DISCONN_COMPLETE, &flt);
 	hci_filter_set_event(EVT_AUTH_COMPLETE, &flt);
+	hci_filter_set_event(EVT_CONN_REQUEST, &flt);
 	if (setsockopt(dev, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
-		syslog(LOG_ERR, "Can't set filter on hci%d: %s (%d)",
+		error("Can't set filter on hci%d: %s (%d)",
 						hdev, strerror(errno), errno);
 		close(dev);
 		return;
@@ -816,25 +836,27 @@ void start_security_manager(int hdev)
 
 	di = malloc(sizeof(*di));
 	if (!di) {
-		syslog(LOG_ERR, "Can't allocate device info buffer: %s (%d)",
+		error("Can't allocate device info buffer: %s (%d)",
 							strerror(errno), errno);
 		close(dev);
 		return;
 	}
 
-	di->dev_id = hdev;
-	if (ioctl(dev, HCIGETDEVINFO, (void *)di)) {
-		syslog(LOG_ERR, "Can't get device info: %s (%d)",
+	if (hci_devinfo(hdev, di) < 0) {
+		error("Can't get device info: %s (%d)",
 							strerror(errno), errno);
 		close(dev);
+		free(di);
 		return;
 	}
 
 	chan = g_io_channel_unix_new(dev);
-	g_io_add_watch(chan, G_IO_IN | G_IO_NVAL | G_IO_HUP | G_IO_ERR,
-			io_security_event, (void *) di);
-
-	io_chan[hdev] = chan;
+	g_io_channel_set_close_on_unref(chan, TRUE);
+	io_data[hdev].watch_id = g_io_add_watch_full(chan, G_PRIORITY_HIGH,
+						G_IO_IN | G_IO_NVAL | G_IO_HUP | G_IO_ERR,
+						io_security_event, di, (GDestroyNotify)free);
+	io_data[hdev].channel = chan;
+	io_data[hdev].pin_length = -1;
 
 	if (hci_test_bit(HCI_RAW, &di->flags))
 		return;
@@ -848,28 +870,21 @@ void start_security_manager(int hdev)
 
 void stop_security_manager(int hdev)
 {
-	GIOChannel *chan = io_chan[hdev];
+	GIOChannel *chan = io_data[hdev].channel;
 
 	if (!chan)
 		return;
 
-	syslog(LOG_INFO, "Stoping security manager %d", hdev);
+	info("Stopping security manager %d", hdev);
 
-	/* this is a bit sneaky. closing the fd will cause the event
-	   loop to call us right back with G_IO_NVAL set, at which
-	   point we will see it and clean things up */
-	close(g_io_channel_unix_get_fd(chan));
-	io_chan[hdev] = NULL;
+	g_io_remove_watch(io_data[hdev].watch_id);
+	g_io_channel_unref(io_data[hdev].channel);
+	io_data[hdev].watch_id = -1;
+	io_data[hdev].channel = NULL;
+	io_data[hdev].pin_length = -1;
 }
 
 void init_security_data(void)
 {
-	/* Set local PIN code */
-	if (read_default_pin_code() < 0) {
-		strcpy((char *) hcid.pin_code, "BlueZ");
-		hcid.pin_len = 5;
-	}
-
 	pairing = hcid.pairing;
-	return;
 }
