@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -47,6 +48,10 @@
 
 #include "hidd.h"
 
+#ifdef NEED_PPOLL
+#include "ppoll.h"
+#endif
+
 enum {
 	NONE,
 	SHOW,
@@ -231,7 +236,7 @@ static int request_encryption(bdaddr_t *src, bdaddr_t *dst)
 	return err;
 }
 
-static int create_device(int ctl, int csk, int isk, uint8_t subclass, int nosdp, int nocheck, int encrypt, int timeout)
+static int create_device(int ctl, int csk, int isk, uint8_t subclass, int nosdp, int nocheck, int bootonly, int encrypt, int timeout)
 {
 	struct hidp_connadd_req req;
 	struct sockaddr_l2 addr;
@@ -266,8 +271,12 @@ static int create_device(int ctl, int csk, int isk, uint8_t subclass, int nosdp,
 	if (!err)
 		goto create;
 
-	if (!nocheck)
-		return -1;
+	if (!nocheck) {
+		ba2str(&dst, bda);
+		syslog(LOG_ERR, "Rejected connection from unknown device %s\n", bda);
+		/* Return no error to avoid run_server() complaining too */
+		return 0;
+	}
 
 	if (!nosdp) {
 		err = get_sdp_device_info(&src, &dst, &req);
@@ -310,6 +319,11 @@ create:
 			syslog(LOG_ERR, "Encryption for %s failed", bda);
 	}
 
+	if (bootonly) {
+		req.rd_size = 0;
+		req.flags |= (1 << HIDP_BOOT_PROTOCOL_MODE);
+	}
+
 	err = ioctl(ctl, HIDPCONNADD, &req);
 
 error:
@@ -319,12 +333,20 @@ error:
 	return err;
 }
 
-static void run_server(int ctl, int csk, int isk, uint8_t subclass, int nosdp, int nocheck, int encrypt, int timeout)
+static void run_server(int ctl, int csk, int isk, uint8_t subclass, int nosdp, int nocheck, int bootonly, int encrypt, int timeout)
 {
 	struct pollfd p[2];
+	sigset_t sigs;
 	short events;
 	int err, ncsk, nisk;
 
+	sigfillset(&sigs);
+	sigdelset(&sigs, SIGCHLD);
+	sigdelset(&sigs, SIGPIPE);
+	sigdelset(&sigs, SIGTERM);
+	sigdelset(&sigs, SIGINT);
+	sigdelset(&sigs, SIGHUP);
+
 	p[0].fd = csk;
 	p[0].events = POLLIN | POLLERR | POLLHUP;
 
@@ -335,8 +357,7 @@ static void run_server(int ctl, int csk, int isk, uint8_t subclass, int nosdp, i
 		p[0].revents = 0;
 		p[1].revents = 0;
 
-		err = poll(p, 2, 100);
-		if (err <= 0)
+		if (ppoll(p, 2, NULL, &sigs) < 1)
 			continue;
 
 		events = p[0].revents | p[1].revents;
@@ -345,7 +366,7 @@ static void run_server(int ctl, int csk, int isk, uint8_t subclass, int nosdp, i
 			ncsk = l2cap_accept(csk, NULL);
 			nisk = l2cap_accept(isk, NULL);
 
-			err = create_device(ctl, ncsk, nisk, subclass, nosdp, nocheck, encrypt, timeout);
+			err = create_device(ctl, ncsk, nisk, subclass, nosdp, nocheck, bootonly, encrypt, timeout);
 			if (err < 0)
 				syslog(LOG_ERR, "HID create error %d (%s)",
 						errno, strerror(errno));
@@ -409,18 +430,20 @@ static void do_show(int ctl)
 	}
 }
 
-static void do_connect(int ctl, bdaddr_t *src, bdaddr_t *dst, uint8_t subclass, int fakehid, int encrypt, int timeout)
+static void do_connect(int ctl, bdaddr_t *src, bdaddr_t *dst, uint8_t subclass, int fakehid, int bootonly, int encrypt, int timeout)
 {
 	struct hidp_connadd_req req;
 	uint16_t uuid = HID_SVCLASS_ID;
 	uint8_t channel = 0;
+	char name[256];
 	int csk, isk, err;
 
 	memset(&req, 0, sizeof(req));
 
 	err = get_sdp_device_info(src, dst, &req);
 	if (err < 0 && fakehid)
-		err = get_alternate_device_info(src, dst, &uuid, &channel);
+		err = get_alternate_device_info(src, dst,
+				&uuid, &channel, name, sizeof(name) - 1);
 
 	if (err < 0) {
 		perror("Can't get device information");
@@ -433,12 +456,35 @@ static void do_connect(int ctl, bdaddr_t *src, bdaddr_t *dst, uint8_t subclass,
 		goto connect;
 
 	case SERIAL_PORT_SVCLASS_ID:
-		epox_presenter(src, dst, channel);
+		if (subclass == 0x40 || !strcmp(name, "Cable Replacement")) {
+			if (epox_presenter(src, dst, channel) < 0) {
+				close(ctl);
+				exit(1);
+			}
+			break;
+		}
+		if (subclass == 0x1f || !strcmp(name, "SPP slave")) {
+			if (jthree_keyboard(src, dst, channel) < 0) {
+				close(ctl);
+				exit(1);
+			}
+			break;
+		}
+		if (subclass == 0x02 || !strcmp(name, "Serial Port")) {
+			if (celluon_keyboard(src, dst, channel) < 0) {
+				close(ctl);
+				exit(1);
+			}
+			break;
+		}
 		break;
 
 	case HEADSET_SVCLASS_ID:
 	case HANDSFREE_SVCLASS_ID:
-		headset_presenter(src, dst, channel);
+		if (headset_presenter(src, dst, channel) < 0) {
+			close(ctl);
+			exit(1);
+		}
 		break;
 	}
 
@@ -460,7 +506,7 @@ connect:
 		exit(1);
 	}
 
-	err = create_device(ctl, csk, isk, subclass, 1, 1, encrypt, timeout);
+	err = create_device(ctl, csk, isk, subclass, 1, 1, bootonly, encrypt, timeout);
 	if (err < 0) {
 		fprintf(stderr, "HID create error %d (%s)\n",
 						errno, strerror(errno));
@@ -472,7 +518,7 @@ connect:
 	}
 }
 
-static void do_search(int ctl, bdaddr_t *bdaddr, uint8_t subclass, int fakehid, int encrypt, int timeout)
+static void do_search(int ctl, bdaddr_t *bdaddr, uint8_t subclass, int fakehid, int bootonly, int encrypt, int timeout)
 {
 	inquiry_info *info = NULL;
 	bdaddr_t src, dst;
@@ -503,7 +549,7 @@ static void do_search(int ctl, bdaddr_t *bdaddr, uint8_t subclass, int fakehid,
 			ba2str(&dst, addr);
 
 			printf("\tConnecting to device %s\n", addr);
-			do_connect(ctl, &src, &dst, subclass, fakehid, encrypt, timeout);
+			do_connect(ctl, &src, &dst, subclass, fakehid, bootonly, encrypt, timeout);
 		}
 	}
 
@@ -512,12 +558,14 @@ static void do_search(int ctl, bdaddr_t *bdaddr, uint8_t subclass, int fakehid,
 
 	for (i = 0; i < num_rsp; i++) {
 		memcpy(class, (info+i)->dev_class, 3);
-		if (class[0] == 0x00 && class[1] == 0x40 && class[2] == 0x00) {
+		if ((class[0] == 0x00 && class[2] == 0x00 && 
+				(class[1] == 0x40 || class[1] == 0x1f)) ||
+				(class[0] == 0x10 && class[1] == 0x02 && class[2] == 0x40)) {
 			bacpy(&dst, &(info+i)->bdaddr);
 			ba2str(&dst, addr);
 
 			printf("\tConnecting to device %s\n", addr);
-			do_connect(ctl, &src, &dst, subclass, 1, 0, timeout);
+			do_connect(ctl, &src, &dst, subclass, 1, bootonly, 0, timeout);
 		}
 	}
 
@@ -608,6 +656,8 @@ static struct option main_options[] = {
 	{ "encrypt",	0, 0, 'E' },
 	{ "nosdp",	0, 0, 'D' },
 	{ "nocheck",	0, 0, 'Z' },
+	{ "bootonly",	0, 0, 'B' },
+	{ "hidonly",	0, 0, 'H' },
 	{ "show",	0, 0, 'l' },
 	{ "list",	0, 0, 'l' },
 	{ "server",	0, 0, 'd' },
@@ -632,13 +682,13 @@ int main(int argc, char *argv[])
 	uint8_t subclass = 0x00;
 	char addr[18];
 	int log_option = LOG_NDELAY | LOG_PID;
-	int opt, fd, ctl, csk, isk;
-	int mode = SHOW, daemon = 1, nosdp = 0, nocheck = 0;
+	int opt, ctl, csk, isk;
+	int mode = SHOW, detach = 1, nosdp = 0, nocheck = 0, bootonly = 0;
 	int fakehid = 1, encrypt = 0, timeout = 30, lm = 0;
 
 	bacpy(&bdaddr, BDADDR_ANY);
 
-	while ((opt = getopt_long(argc, argv, "+i:nt:b:MEDZldsc:k:Ku:h", main_options, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "+i:nt:b:MEDZBHldsc:k:Ku:h", main_options, NULL)) != -1) {
 		switch(opt) {
 		case 'i':
 			if (!strncasecmp(optarg, "hci", 3))
@@ -647,7 +697,7 @@ int main(int argc, char *argv[])
 				str2ba(optarg, &bdaddr);
 			break;
 		case 'n':
-			daemon = 0;
+			detach = 0;
 			break;
 		case 't':
 			timeout = atoi(optarg);
@@ -670,6 +720,12 @@ int main(int argc, char *argv[])
 		case 'Z':
 			nocheck = 1;
 			break;
+		case 'B':
+			bootonly = 1;
+			break;
+		case 'H':
+			fakehid = 0;
+			break;
 		case 'l':
 			mode = SHOW;
 			break;
@@ -731,12 +787,12 @@ int main(int argc, char *argv[])
 		break;
 
 	case SEARCH:
-		do_search(ctl, &bdaddr, subclass, fakehid, encrypt, timeout);
+		do_search(ctl, &bdaddr, subclass, fakehid, bootonly, encrypt, timeout);
 		close(ctl);
 		exit(0);
 
 	case CONNECT:
-		do_connect(ctl, &bdaddr, &dev, subclass, fakehid, encrypt, timeout);
+		do_connect(ctl, &bdaddr, &dev, subclass, fakehid, bootonly, encrypt, timeout);
 		close(ctl);
 		exit(0);
 
@@ -751,16 +807,11 @@ int main(int argc, char *argv[])
 		exit(0);
 	}
 
-	if (daemon) {
-		if (fork())
-			exit(0);
-
-		fd = open("/dev/null", O_RDWR);
-		dup2(fd, 0); dup2(fd, 1); dup2(fd, 2);
-		close(fd);
-
-		setsid();
-		chdir("/");
+        if (detach) {
+		if (daemon(0, 0)) {
+			perror("Can't start daemon");
+        	        exit(1);
+		}
 	} else
 		log_option |= LOG_PERROR;
 
@@ -784,7 +835,7 @@ int main(int argc, char *argv[])
 	sigaction(SIGCHLD, &sa, NULL);
 	sigaction(SIGPIPE, &sa, NULL);
 
-	run_server(ctl, csk, isk, subclass, nosdp, nocheck, encrypt, timeout);
+	run_server(ctl, csk, isk, subclass, nosdp, nocheck, bootonly, encrypt, timeout);
 
 	syslog(LOG_INFO, "Exit");
 
