@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@
 #include <malloc.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/param.h>
 #include <sys/socket.h>
 
 #include <bluetooth/bluetooth.h>
@@ -75,7 +76,7 @@ static int store_device_info(const bdaddr_t *src, const bdaddr_t *dst, struct hi
 	int i, size;
 
 	ba2str(src, addr);
-	snprintf(filename, PATH_MAX, "%s/%s/hidd", STORAGEDIR, addr);
+	create_name(filename, PATH_MAX, STORAGEDIR, addr, "hidd");
 
 	size = 15 + 3 + 3 + 5 + (req->rd_size * 2) + 1 + 9 + strlen(req->name) + 2;
 	str = malloc(size);
@@ -116,7 +117,7 @@ int get_stored_device_info(const bdaddr_t *src, const bdaddr_t *dst, struct hidp
 	memset(desc, 0, 4096);
 
 	ba2str(src, addr);
-	snprintf(filename, PATH_MAX, "%s/%s/hidd", STORAGEDIR, addr);
+	create_name(filename, PATH_MAX, STORAGEDIR, addr, "hidd");
 
 	ba2str(dst, addr);
 	str = textfile_get(filename, addr);
@@ -270,9 +271,10 @@ int get_sdp_device_info(const bdaddr_t *src, const bdaddr_t *dst, struct hidp_co
 	return 0;
 }
 
-int get_alternate_device_info(const bdaddr_t *src, const bdaddr_t *dst, uint16_t *uuid, uint8_t *channel)
+int get_alternate_device_info(const bdaddr_t *src, const bdaddr_t *dst, uint16_t *uuid, uint8_t *channel, char *name, size_t len)
 {
-	uint16_t attr = SDP_ATTR_PROTO_DESC_LIST;
+	uint16_t attr1 = SDP_ATTR_PROTO_DESC_LIST;
+	uint16_t attr2 = SDP_ATTR_SVCNAME_PRIMARY;
 	sdp_session_t *s;
 	sdp_list_t *search, *attrid, *rsp;
 	uuid_t svclass;
@@ -284,7 +286,8 @@ int get_alternate_device_info(const bdaddr_t *src, const bdaddr_t *dst, uint16_t
 
 	sdp_uuid16_create(&svclass, HEADSET_SVCLASS_ID);
 	search = sdp_list_append(NULL, &svclass);
-	attrid = sdp_list_append(NULL, &attr);
+	attrid = sdp_list_append(NULL, &attr1);
+	attrid = sdp_list_append(attrid, &attr2);
 
 	err = sdp_service_search_attr_req(s, search,
 					SDP_ATTR_REQ_INDIVIDUAL, attrid, &rsp);
@@ -295,7 +298,8 @@ int get_alternate_device_info(const bdaddr_t *src, const bdaddr_t *dst, uint16_t
 	if (err <= 0) {
 		sdp_uuid16_create(&svclass, SERIAL_PORT_SVCLASS_ID);
 		search = sdp_list_append(NULL, &svclass);
-		attrid = sdp_list_append(NULL, &attr);
+		attrid = sdp_list_append(NULL, &attr1);
+		attrid = sdp_list_append(attrid, &attr2);
 
 		err = sdp_service_search_attr_req(s, search,
 					SDP_ATTR_REQ_INDIVIDUAL, attrid, &rsp);
@@ -321,6 +325,8 @@ int get_alternate_device_info(const bdaddr_t *src, const bdaddr_t *dst, uint16_t
 		sdp_record_t *rec = (sdp_record_t *) rsp->data;
 		sdp_list_t *protos;
 
+		sdp_get_service_name(rec, name, len);
+
 		if (!sdp_get_access_protos(rec, &protos)) {
 			uint8_t ch = sdp_get_proto_port(protos, RFCOMM_UUID);
 			if (ch > 0) {
