@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -48,19 +49,20 @@
 
 #include "pand.h"
 
-static uint16_t role = BNEP_SVC_PANU;	/* Local role (ie service) */
-static uint16_t service = BNEP_SVC_NAP;	/* Remote service */
+#ifdef NEED_PPOLL
+#include "ppoll.h"
+#endif
 
-static int  detach = 1;
-static int  persist;
-static int  use_sdp = 1;
-static int  use_cache;
-static int  auth;
-static int  encrypt;
-static int  secure;
-static int  master;
-static int  cleanup;
-static int  search_duration = 10;
+static uint16_t role    = BNEP_SVC_PANU;	/* Local role (ie service) */
+static uint16_t service = BNEP_SVC_NAP;		/* Remote service */
+
+static int detach = 1;
+static int persist;
+static int use_sdp = 1;
+static int use_cache;
+static int link_mode = 0;
+static int cleanup;
+static int search_duration = 10;
 
 static struct {
 	int      valid;
@@ -70,10 +72,12 @@ static struct {
 
 static char netdev[16] = "bnep%d";
 static char *pidfile = NULL;
+static char *devupcmd = NULL;
+
 static bdaddr_t src_addr = *BDADDR_ANY;
 static int src_dev = -1;
 
-volatile int terminate;
+static volatile int terminate;
 
 static void do_kill(char *dst);
 
@@ -87,12 +91,13 @@ enum {
 
 static void run_devup(char *dev, char *dst, int sk, int nsk)
 {
-	char *argv[4], prog[40];
+	char *argv[4];
 	struct sigaction sa;
 
-	sprintf(prog, "%s/%s", PAND_CONFIG_DIR, PAND_DEVUP_CMD);
+	if (!devupcmd)
+		return;
 
-	if (access(prog, R_OK | X_OK))
+	if (access(devupcmd, R_OK | X_OK))
 		return;
 
 	if (fork())
@@ -103,17 +108,19 @@ static void run_devup(char *dev, char *dst, int sk, int nsk)
 
 	if (nsk >= 0)
 		close(nsk);
-	
+
 	memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = SIG_DFL;
 	sigaction(SIGCHLD, &sa, NULL);
 	sigaction(SIGPIPE, &sa, NULL);
 
-	argv[0] = prog;
+	argv[0] = devupcmd;
 	argv[1] = dev;
 	argv[2] = dst;
 	argv[3] = NULL;
-	execv(prog, argv);
+
+	execv(devupcmd, argv);
+
 	exit(1);
 }
 
@@ -131,7 +138,7 @@ static int do_listen(void)
 	sk = socket(AF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP);
 	if (sk < 0) {
 		syslog(LOG_ERR, "Cannot create L2CAP socket. %s(%d)",
-				strerror(errno), errno);
+						strerror(errno), errno);
 		return -1;
 	}
 
@@ -141,7 +148,8 @@ static int do_listen(void)
 	l2a.l2_psm = htobs(BNEP_PSM);
 
 	if (bind(sk, (struct sockaddr *) &l2a, sizeof(l2a))) {
-		syslog(LOG_ERR, "Bind failed. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Bind failed. %s(%d)",
+						strerror(errno), errno);
 		return -1;
 	}
 
@@ -150,30 +158,22 @@ static int do_listen(void)
 	olen = sizeof(l2o);
 	if (getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &l2o, &olen) < 0) {
 		syslog(LOG_ERR, "Failed to get L2CAP options. %s(%d)",
-				strerror(errno), errno);
+						strerror(errno), errno);
 		return -1;
 	}
 
 	l2o.imtu = l2o.omtu = BNEP_MTU;
 	if (setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &l2o, sizeof(l2o)) < 0) {
 		syslog(LOG_ERR, "Failed to set L2CAP options. %s(%d)",
-				strerror(errno), errno);
+						strerror(errno), errno);
 		return -1;
 	}
 
 	/* Set link mode */
-	lm = 0;
-	if (master)
-		lm |= L2CAP_LM_MASTER;
-	if (auth)
-		lm |= L2CAP_LM_AUTH;
-	if (encrypt)
-		lm |= L2CAP_LM_ENCRYPT;
-	if (secure)
-		lm |= L2CAP_LM_SECURE;
-
+	lm = link_mode;
 	if (lm && setsockopt(sk, SOL_L2CAP, L2CAP_LM, &lm, sizeof(lm)) < 0) {
-		syslog(LOG_ERR, "Failed to set link mode. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Failed to set link mode. %s(%d)",
+						strerror(errno), errno);
 		return -1;
 	}
 
@@ -181,10 +181,13 @@ static int do_listen(void)
 
 	while (!terminate) {
 		socklen_t alen = sizeof(l2a);
+		char devname[16];
 		int nsk;
+
 		nsk = accept(sk, (struct sockaddr *) &l2a, &alen);
 		if (nsk < 0) {
-			syslog(LOG_ERR, "Accept failed. %s(%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Accept failed. %s(%d)",
+						strerror(errno), errno);
 			continue;
 		}
 
@@ -192,22 +195,27 @@ static int do_listen(void)
 		case 0:
 			break;
 		case -1:
-			syslog(LOG_ERR, "Fork failed. %s(%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Fork failed. %s(%d)",
+						strerror(errno), errno);
 		default:
 			close(nsk);
 			continue;
 		}
 
-		if (!bnep_accept_connection(nsk, role, netdev)) {
+		strncpy(devname, netdev, 16);
+		devname[15] = '\0';
+
+		if (!bnep_accept_connection(nsk, role, devname)) {
 			char str[40];
 			ba2str(&l2a.l2_bdaddr, str);
 
-			syslog(LOG_INFO, "New connection from %s %s", str, netdev);
+			syslog(LOG_INFO, "New connection from %s %s",
+								str, netdev);
 
-			run_devup(netdev, str, sk, nsk);
+			run_devup(devname, str, sk, nsk);
 		} else {
 			syslog(LOG_ERR, "Connection failed. %s(%d)",
-					strerror(errno), errno);
+						strerror(errno), errno);
 		}
 
 		close(nsk);
@@ -223,17 +231,25 @@ static int do_listen(void)
 static int w4_hup(int sk)
 {
 	struct pollfd pf;
+	sigset_t sigs;
 	int n;
 
+	sigfillset(&sigs);
+	sigdelset(&sigs, SIGCHLD);
+	sigdelset(&sigs, SIGPIPE);
+	sigdelset(&sigs, SIGTERM);
+	sigdelset(&sigs, SIGINT);
+	sigdelset(&sigs, SIGHUP);
+
 	while (!terminate) {
 		pf.fd = sk;
 		pf.events = POLLERR | POLLHUP;
-		n = poll(&pf, 1, -1);
+		n = ppoll(&pf, 1, NULL, &sigs);
 		if (n < 0) {
 			if (errno == EINTR || errno == EAGAIN)
 				continue;
 			syslog(LOG_ERR, "Poll failed. %s(%d)",
-					strerror(errno), errno);
+						strerror(errno), errno);
 			return 1;
 		}
 
@@ -269,7 +285,7 @@ static int create_connection(char *dst, bdaddr_t *bdaddr)
 	sk = socket(AF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP);
 	if (sk < 0) {
 		syslog(LOG_ERR, "Cannot create L2CAP socket. %s(%d)",
-				strerror(errno), errno);
+						strerror(errno), errno);
 		return -1;
 	}
 
@@ -285,8 +301,8 @@ static int create_connection(char *dst, bdaddr_t *bdaddr)
 	bacpy(&l2a.l2_bdaddr, &src_addr);
 
 	if (bind(sk, (struct sockaddr *) &l2a, sizeof(l2a)))
-		syslog(LOG_ERR, "Bind failed. %s(%d)", 
-				strerror(errno), errno);
+		syslog(LOG_ERR, "Bind failed. %s(%d)",
+						strerror(errno), errno);
 
 	memset(&l2a, 0, sizeof(l2a));
 	l2a.l2_family = AF_BLUETOOTH;
@@ -312,7 +328,7 @@ static int create_connection(char *dst, bdaddr_t *bdaddr)
 		r = 0;
 	} else {
 		syslog(LOG_ERR, "Connect to %s failed. %s(%d)",
-				dst, strerror(errno), errno);
+						dst, strerror(errno), errno);
 		r = 1;
 	}
 
@@ -365,7 +381,8 @@ static int do_connect(void)
 		ii = NULL;
 		n  = hci_inquiry(src_dev, search_duration, 0, NULL, &ii, 0);
 		if (n < 0) {
-			syslog(LOG_ERR, "Inquiry failed. %s(%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Inquiry failed. %s(%d)",
+						strerror(errno), errno);
 			continue;
 		}
 
@@ -406,29 +423,30 @@ static void do_kill(char *dst)
 		bnep_kill_all_connections();
 }
 
-void sig_hup(int sig)
+static void sig_hup(int sig)
 {
 	return;
 }
 
-void sig_term(int sig)
+static void sig_term(int sig)
 {
 	terminate = 1;
 }
 
-int write_pidfile(void)
+static int write_pidfile(void)
 {
 	int fd;
 	FILE *f;
 	pid_t pid;
 
-	do { 
+	do {
 		fd = open(pidfile, O_WRONLY|O_TRUNC|O_CREAT|O_EXCL, 0644);
 		if (fd == -1) {
 			/* Try to open the file for read. */
 			fd = open(pidfile, O_RDONLY);
-			if(fd == -1) {
-				syslog(LOG_ERR, "Could not read old pidfile: %s(%d)", strerror(errno), errno);
+			if (fd < 0) {
+				syslog(LOG_ERR, "Could not read old pidfile: %s(%d)",
+							strerror(errno), errno);
 				return -1;
 			}
 			
@@ -440,13 +458,15 @@ int write_pidfile(void)
 			 */
 			f = fdopen(fd, "r");
 			if (!f) {
-				syslog(LOG_ERR, "Could not fdopen old pidfile: %s(%d)", strerror(errno), errno);
+				syslog(LOG_ERR, "Could not fdopen old pidfile: %s(%d)",
+							strerror(errno), errno);
 				close(fd);
 				return -1;
 			}
 
 			pid = 0;
-			fscanf(f, "%d", &pid);
+			if (fscanf(f, "%d", &pid) != 1)
+				pid = 0;
 			fclose(f);
 
 			if (pid) {
@@ -469,17 +489,18 @@ int write_pidfile(void)
 
 	f = fdopen(fd, "w");
 	if (!f) {
-		syslog(LOG_ERR, "Could not fdopen new pidfile: %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Could not fdopen new pidfile: %s(%d)",
+						strerror(errno), errno);
 		close(fd);
 		unlink(pidfile);
 		return -1;
 	}
+
 	fprintf(f, "%d\n", getpid());
 	fclose(f);
+
 	return 0;
 }
-		
-
 
 static struct option main_lopts[] = {
 	{ "help",     0, 0, 'h' },
@@ -503,11 +524,12 @@ static struct option main_lopts[] = {
 	{ "master",   0, 0, 'M' },
 	{ "cache",    0, 0, 'C' },
 	{ "pidfile",  1, 0, 'P' },
+	{ "devup",    1, 0, 'u' },
 	{ "autozap",  0, 0, 'z' },
 	{ 0, 0, 0, 0 }
 };
 
-static char main_sopts[] = "hsc:k:Kr:d:e:i:lnp::DQ::AESMC::P:z";
+static char main_sopts[] = "hsc:k:Kr:d:e:i:lnp::DQ::AESMC::P:u:z";
 
 static char main_help[] = 
 	"Bluetooth PAN daemon version " VERSION " \n"
@@ -533,9 +555,10 @@ static char main_help[] =
 	"\t--nodetach -n             Do not become a daemon\n"
 	"\t--persist -p[interval]    Persist mode\n"
 	"\t--cache -C[valid]         Cache addresses\n"
-	"\t--pidfile -P <pidfile>    Create PID file\n";
+	"\t--pidfile -P <pidfile>    Create PID file\n"
+	"\t--devup -u <script>       Script to run when interface comes up\n";
 
-int main(int argc, char **argv)
+int main(int argc, char *argv[])
 {
 	char *dst = NULL, *src = NULL;
 	struct sigaction sa;
@@ -592,23 +615,24 @@ int main(int argc, char **argv)
 			break;
 
 		case 'A':
-			auth = 1;
+			link_mode |= L2CAP_LM_AUTH;
 			break;
 
 		case 'E':
-			encrypt = 1;
+			link_mode |= L2CAP_LM_ENCRYPT;
 			break;
 
 		case 'S':
-			secure = 1;
+			link_mode |= L2CAP_LM_SECURE;
 			break;
 
 		case 'M':
-			master = 1;
+			link_mode |= L2CAP_LM_MASTER;
 			break;
 
 		case 'e':
-			strcpy(netdev, optarg);
+			strncpy(netdev, optarg, 16);
+			netdev[15] = '\0';
 			break;
 
 		case 'n':
@@ -633,6 +657,10 @@ int main(int argc, char **argv)
 			pidfile = strdup(optarg);
 			break;
 
+		case 'u':
+			devupcmd = strdup(optarg);
+			break;
+
 		case 'z':
 			cleanup = 1;
 			break;
@@ -680,18 +708,9 @@ int main(int argc, char **argv)
 	sigaction(SIGTERM, &sa, NULL);
 	sigaction(SIGINT,  &sa, NULL);
 
-	if (detach) {
-		if (fork()) exit(0);
-
-		/* Direct stdin,stdout,stderr to '/dev/null' */
-		{
-			int fd = open("/dev/null", O_RDWR);
-			dup2(fd, 0); dup2(fd, 1); dup2(fd, 2);
-			close(fd);
-		}
-
-		setsid();
-		chdir("/");
+	if (detach && daemon(0, 0)) {
+		perror("Can't start daemon");
+		exit(1);
 	}
 
 	openlog("pand", LOG_PID | LOG_NDELAY | LOG_PERROR, LOG_DAEMON);
@@ -700,7 +719,8 @@ int main(int argc, char **argv)
 	if (src) {
 		src_dev = hci_devid(src);
 		if (src_dev < 0 || hci_devba(src_dev, &src_addr) < 0) {
-			syslog(LOG_ERR, "Invalid source. %s(%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Invalid source. %s(%d)",
+						strerror(errno), errno);
 			return -1;
 		}
 	}
