@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -47,6 +48,10 @@
 
 #include "kword.h"
 
+#ifdef NEED_PPOLL
+#include "ppoll.h"
+#endif
+
 static char *rfcomm_config_file = NULL;
 static int rfcomm_raw_tty = 0;
 
@@ -150,7 +155,8 @@ static int create_dev(int ctl, int dev, uint32_t flags, bdaddr_t *bdaddr, int ar
 	bacpy(&req.src, bdaddr);
 
 	if (argc < 2) {
-		if ((err = rfcomm_read_config(rfcomm_config_file)) < 0) {
+		err = rfcomm_read_config(rfcomm_config_file);
+		if (err < 0) {
 			perror("Can't open RFCOMM config file");
 			return err;
 		}
@@ -172,7 +178,10 @@ static int create_dev(int ctl, int dev, uint32_t flags, bdaddr_t *bdaddr, int ar
 			req.channel = 1;
 	}
 
-	if ((err = ioctl(ctl, RFCOMMCREATEDEV, &req)) < 0 )
+	err = ioctl(ctl, RFCOMMCREATEDEV, &req);
+	if (err == EOPNOTSUPP)
+		fprintf(stderr, "RFCOMM TTY support not available\n");
+	else if (err < 0)
 		perror("Can't create device");
 
 	return err;
@@ -183,7 +192,8 @@ static int create_all(int ctl)
 	struct rfcomm_dev_req req;
 	int i, err;
 
-	if ((err = rfcomm_read_config(rfcomm_config_file)) < 0) {
+	err = rfcomm_read_config(rfcomm_config_file);
+	if (err < 0) {
 		perror("Can't open RFCOMM config file");
 		return err;
 	}
@@ -214,7 +224,8 @@ static int release_dev(int ctl, int dev, uint32_t flags)
 	memset(&req, 0, sizeof(req));
 	req.dev_id = dev;
 
-	if ((err = ioctl(ctl, RFCOMMRELEASEDEV, &req)) < 0 )
+	err = ioctl(ctl, RFCOMMRELEASEDEV, &req);
+	if (err < 0)
 		perror("Can't release device");
 
 	return err;
@@ -253,9 +264,10 @@ static void cmd_connect(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **arg
 	struct termios ti;
 	struct sigaction sa;
 	struct pollfd p;
+	sigset_t sigs;
 	socklen_t alen;
 	char dst[18], devname[MAXPATHLEN];
-	int sk, fd, try = 3;
+	int sk, fd, try = 30;
 
 	laddr.rc_family = AF_BLUETOOTH;
 	bacpy(&laddr.rc_bdaddr, bdaddr);
@@ -285,7 +297,8 @@ static void cmd_connect(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **arg
 			raddr.rc_channel = 1;
 	}
 
-	if ((sk = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM)) < 0) {
+	sk = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);
+	if (sk < 0) {
 		perror("Can't create RFCOMM socket");
 		return;
 	}
@@ -317,7 +330,8 @@ static void cmd_connect(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **arg
 	bacpy(&req.dst, &raddr.rc_bdaddr);
 	req.channel = raddr.rc_channel;
 
-	if ((dev = ioctl(sk, RFCOMMCREATEDEV, &req)) < 0) {
+	dev = ioctl(sk, RFCOMMCREATEDEV, &req);
+	if (dev < 0) {
 		perror("Can't create RFCOMM TTY");
 		close(sk);
 		return;
@@ -329,7 +343,7 @@ static void cmd_connect(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **arg
 		if ((fd = open(devname, O_RDONLY | O_NOCTTY)) < 0) {
 			if (try--) {
 				snprintf(devname, MAXPATHLEN - 1, "/dev/rfcomm%d", dev);
-				sleep(1);
+				usleep(100);
 				continue;
 			}
 			perror("Can't open RFCOMM device");
@@ -370,12 +384,19 @@ static void cmd_connect(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **arg
 	sa.sa_handler = sig_hup;
 	sigaction(SIGHUP, &sa, NULL);
 
+	sigfillset(&sigs);
+	sigdelset(&sigs, SIGCHLD);
+	sigdelset(&sigs, SIGPIPE);
+	sigdelset(&sigs, SIGTERM);
+	sigdelset(&sigs, SIGINT);
+	sigdelset(&sigs, SIGHUP);
+
 	p.fd = fd;
 	p.events = POLLERR | POLLHUP;
 
 	while (!__io_canceled) {
 		p.revents = 0;
-		if (poll(&p, 1, 100))
+		if (ppoll(&p, 1, NULL, &sigs) > 0)
 			break;
 	}
 
@@ -391,15 +412,17 @@ static void cmd_listen(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **argv
 	struct termios ti;
 	struct sigaction sa;
 	struct pollfd p;
+	sigset_t sigs;
 	socklen_t alen;
 	char dst[18], devname[MAXPATHLEN];
-	int sk, nsk, fd, try = 3;
+	int sk, nsk, fd, try = 30;
 
 	laddr.rc_family = AF_BLUETOOTH;
 	bacpy(&laddr.rc_bdaddr, bdaddr);
 	laddr.rc_channel = (argc < 2) ? 1 : atoi(argv[1]);
 
-	if ((sk = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM)) < 0) {
+	sk = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);
+	if (sk < 0) {
 		perror("Can't create RFCOMM socket");
 		return;
 	}
@@ -432,7 +455,8 @@ static void cmd_listen(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **argv
 	bacpy(&req.dst, &raddr.rc_bdaddr);
 	req.channel = raddr.rc_channel;
 
-	if ((dev = ioctl(nsk, RFCOMMCREATEDEV, &req)) < 0) {
+	dev = ioctl(nsk, RFCOMMCREATEDEV, &req);
+	if (dev < 0) {
 		perror("Can't create RFCOMM TTY");
 		close(sk);
 		return;
@@ -444,7 +468,7 @@ static void cmd_listen(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **argv
 		if ((fd = open(devname, O_RDONLY | O_NOCTTY)) < 0) {
 			if (try--) {
 				snprintf(devname, MAXPATHLEN - 1, "/dev/rfcomm%d", dev);
-				sleep(1);
+				usleep(100);
 				continue;
 			}
 			perror("Can't open RFCOMM device");
@@ -486,12 +510,19 @@ static void cmd_listen(int ctl, int dev, bdaddr_t *bdaddr, int argc, char **argv
 	sa.sa_handler = sig_hup;
 	sigaction(SIGHUP, &sa, NULL);
 
+	sigfillset(&sigs);
+	sigdelset(&sigs, SIGCHLD);
+	sigdelset(&sigs, SIGPIPE);
+	sigdelset(&sigs, SIGTERM);
+	sigdelset(&sigs, SIGINT);
+	sigdelset(&sigs, SIGHUP);
+
 	p.fd = fd;
 	p.events = POLLERR | POLLHUP;
 
 	while (!__io_canceled) {
 		p.revents = 0;
-		if (poll(&p, 1, 100))
+		if (ppoll(&p, 1, NULL, &sigs) > 0)
 			break;
 	}
 
@@ -557,9 +588,11 @@ static void usage(void)
 		"\n");
 
 	printf("Options:\n"
-		"\t-i [hciX|bdaddr]   Local HCI device or BD Address\n"
-		"\t-h, --help         Display help\n"
-		"\t-a                 Show all devices (default)\n"
+		"\t-i [hciX|bdaddr]      Local HCI device or BD Address\n"
+		"\t-h, --help            Display help\n"
+		"\t-r, --raw             Switch TTY into raw mode\n"
+		"\t-f, --config [file]   Specify alternate config file\n" 
+		"\t-a                    Show all devices (default)\n"
 		"\n");
 
 	printf("Commands:\n");
@@ -621,10 +654,16 @@ int main(int argc, char *argv[])
 	argv += optind;
 	optind = 0;
 
-	if (argc < 2)
-		show_all = 1;
+	if (argc < 2) {
+		if (argc != 0) {
+			usage();
+			exit(1);
+		} else
+			show_all = 1;
+	}
 
-	if ((ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_RFCOMM)) < 0 ) {
+	ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_RFCOMM);
+	if (ctl < 0) {
 		perror("Can't open RFCOMM control socket");
 		exit(1);
 	}
