@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2001-2002  Nokia Corporation
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2002-2003  Stephen Crane <steve.crane@rococosoft.com>
  *
  *
@@ -54,11 +54,10 @@
  * sequence. The data type of elements found in the
  * sequence is returned in the reference pDataType
  */
-static int extract_des(uint8_t *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType, uint8_t expectedType)
+static int extract_des(uint8_t *buf, int len, sdp_list_t **svcReqSeq, uint8_t *pDataType, uint8_t expectedType)
 {
 	uint8_t seqType;
-	int data_size = 0;
-	int scanned = sdp_extract_seqtype(buf, &seqType, &data_size);
+	int scanned, data_size = 0;
 	short numberOfElements = 0;
 	int seqlen = 0;
 	sdp_list_t *pSeq = NULL;
@@ -66,6 +65,8 @@ static int extract_des(uint8_t *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType,
 	int status = 0;
 	const uint8_t *p;
 
+	scanned = sdp_extract_seqtype(buf, &seqType, &data_size);
+
 	SDPDBG("Seq type : %d\n", seqType);
 	if (!scanned || (seqType != SDP_SEQ8 && seqType != SDP_SEQ16)) {
 		SDPERR("Unknown seq type \n");
@@ -95,7 +96,7 @@ static int extract_des(uint8_t *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType,
 		case SDP_UINT16:
 			p += sizeof(uint8_t);
 			seqlen += sizeof(uint8_t);
-			pElem = (char *)malloc(sizeof(uint16_t));
+			pElem = malloc(sizeof(uint16_t));
 			sdp_put_unaligned(ntohs(sdp_get_unaligned((uint16_t *)p)), (uint16_t *)pElem);
 			p += sizeof(uint16_t);
 			seqlen += sizeof(uint16_t);
@@ -103,7 +104,7 @@ static int extract_des(uint8_t *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType,
 		case SDP_UINT32:
 			p += sizeof(uint8_t);
 			seqlen += sizeof(uint8_t);
-			pElem = (char *)malloc(sizeof(uint32_t));
+			pElem = malloc(sizeof(uint32_t));
 			sdp_put_unaligned(ntohl(sdp_get_unaligned((uint32_t *)p)), (uint32_t *)pElem);
 			p += sizeof(uint32_t);
 			seqlen += sizeof(uint32_t);
@@ -111,13 +112,15 @@ static int extract_des(uint8_t *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType,
 		case SDP_UUID16:
 		case SDP_UUID32:
 		case SDP_UUID128:
-			pElem = (char *)malloc(sizeof(uuid_t));
+			pElem = malloc(sizeof(uuid_t));
 			status = sdp_uuid_extract(p, (uuid_t *)pElem, &localSeqLength);
 			if (status == 0) {
 				seqlen += localSeqLength;
 				p += localSeqLength;
 			}
 			break;
+		default:
+			return -1;
 		}
 		if (status == 0) {
 			pSeq = sdp_list_append(pSeq, pElem);
@@ -126,7 +129,7 @@ static int extract_des(uint8_t *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType,
 
 			if (seqlen == data_size)
 				break;
-			else if (seqlen > data_size)
+			else if (seqlen > data_size || seqlen > len)
 				return -1;
 		} else
 			free(pElem);
@@ -214,7 +217,7 @@ static int sdp_match_uuid(sdp_list_t *search, sdp_list_t *pattern)
 		// create 128-bit form of the search UUID
 		uuid128 = sdp_uuid_to_uuid128((uuid_t *)data);
 		list = sdp_list_find(pattern, uuid128, sdp_uuid128_cmp);
-		free(uuid128);
+		bt_free(uuid128);
 		if (!list)
 			return 0;
 	}
@@ -228,7 +231,7 @@ static int sdp_match_uuid(sdp_list_t *search, sdp_list_t *pattern)
  */
 static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 {
-	int status = 0, i, plen, mlen;
+	int status = 0, i, plen, mlen, mtu, scanned;
 	sdp_list_t *pattern = NULL;
 	uint16_t expected, actual;
 	uint8_t dtd;
@@ -238,12 +241,13 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 	uint32_t cStateId = 0;
 	short rsp_count = 0;
 	short *pTotalRecordCount, *pCurrentRecordCount;
-	int mtu;
 	uint8_t *pdata = req->buf + sizeof(sdp_pdu_hdr_t);
-	int scanned = extract_des(pdata, &pattern, &dtd, SDP_TYPE_UUID);
 
 	SDPDBG("");
 
+	scanned = extract_des(pdata, req->len - sizeof(sdp_pdu_hdr_t),
+					&pattern, &dtd, SDP_TYPE_UUID);
+
 	if (scanned == -1) {
 		status = SDP_INVALID_SYNTAX;
 		goto done;
@@ -448,6 +452,7 @@ static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, uint8_t dtd, sdp_bu
 			uint16_t attr;
 			uint16_t low = (0xffff0000 & range) >> 16;
 			uint16_t high = 0x0000ffff & range;
+			sdp_data_t *data;
 
 			SDPDBG("attr range : 0x%x\n", range);
 			SDPDBG("Low id : 0x%x\n", low);
@@ -460,11 +465,14 @@ static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, uint8_t dtd, sdp_bu
 				break;
 			}
 			/* (else) sub-range of attributes */
-			for (attr = low; attr <= high; attr++) {
-				sdp_data_t *a = (sdp_data_t *)sdp_data_get(rec, attr);
-				if (a)
-					sdp_append_to_pdu(buf, a);
+			for (attr = low; attr < high; attr++) {
+				data = sdp_data_get(rec, attr);
+				if (data)
+					sdp_append_to_pdu(buf, data);
 			}
+			data = sdp_data_get(rec, high);
+			if (data)
+				sdp_append_to_pdu(buf, data);
 		}
 		free(pdu.data);
 	} else {
@@ -501,7 +509,8 @@ static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	pdata += sizeof(uint16_t);
 
 	/* extract the attribute list */
-	scanned = extract_des(pdata, &seq, &dtd, SDP_TYPE_ANY);
+	scanned = extract_des(pdata, req->len - sizeof(sdp_pdu_hdr_t),
+						&seq, &dtd, SDP_TYPE_ANY);
 	if (scanned == -1) {
 		status = SDP_INVALID_SYNTAX;
 		goto done;
@@ -613,7 +622,8 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 
 	tmpbuf.data = NULL;
 	pdata = req->buf + sizeof(sdp_pdu_hdr_t);
-	scanned = extract_des(pdata, &pattern, &dtd, SDP_TYPE_UUID);
+	scanned = extract_des(pdata, req->len - sizeof(sdp_pdu_hdr_t),
+					&pattern, &dtd, SDP_TYPE_UUID);
 	if (scanned == -1) {
         	status = SDP_INVALID_SYNTAX;
 		goto done;
@@ -629,7 +639,8 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	SDPDBG("Max Attr expected: %d", max);
 
 	/* extract the attribute list */
-	scanned = extract_des(pdata, &seq, &dtd, SDP_TYPE_ANY);
+	scanned = extract_des(pdata, req->len - sizeof(sdp_pdu_hdr_t),
+						&seq, &dtd, SDP_TYPE_ANY);
 	if (scanned == -1) {
 		status = SDP_INVALID_SYNTAX;
 		goto done;
