@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -219,6 +219,7 @@ static int write_snoop(int fd, int type, int incoming, unsigned char *buf, int l
 	struct timeval tv;
 	uint32_t size = len;
 	uint64_t ts;
+	int err;
 
 	if (fd < 0)
 		return -1;
@@ -236,8 +237,8 @@ static int write_snoop(int fd, int type, int incoming, unsigned char *buf, int l
 	if (type == HCI_COMMAND_PKT || type == HCI_EVENT_PKT)
 		pkt.flags |= ntohl(0x02);
 
-	write(fd, &pkt, BTSNOOP_PKT_SIZE);
-	write(fd, buf, size);
+	err = write(fd, &pkt, BTSNOOP_PKT_SIZE);
+	err = write(fd, buf, size);
 
 	return 0;
 }
@@ -502,6 +503,7 @@ static void close_connection(struct vhci_conn *conn)
 					batostr(&conn->dest), conn->handle);
 
 	g_io_channel_close(conn->chan);
+	g_io_channel_unref(conn->chan);
 
 	vconn[conn->handle - 1] = NULL;
 	disconn_complete(conn);
@@ -872,10 +874,12 @@ static gboolean io_acl_data(GIOChannel *chan, GIOCondition cond, gpointer data)
 	unsigned char buf[HCI_MAX_FRAME_SIZE], *ptr;
 	hci_acl_hdr *ah;
 	uint16_t flags;
-	int len, fd;
+	int fd, err, len;
 
-	if (cond & G_IO_NVAL)
+	if (cond & G_IO_NVAL) {
+		g_io_channel_unref(chan);
 		return FALSE;
+	}
 
 	if (cond & G_IO_HUP) {
 		close_connection(conn);
@@ -907,7 +911,7 @@ static gboolean io_acl_data(GIOChannel *chan, GIOCondition cond, gpointer data)
 
 	write_snoop(vdev.dd, HCI_ACLDATA_PKT, 1, buf, len);
 
-	write(vdev.fd, buf, len);
+	err = write(vdev.fd, buf, len);
 
 	return TRUE;
 }
@@ -974,6 +978,7 @@ static gboolean io_hci_data(GIOChannel *chan, GIOCondition cond, gpointer data)
 			return TRUE;
 
 		syslog(LOG_ERR, "Read failed: %s (%d)", strerror(errno), errno);
+		g_io_channel_unref(chan);
 		g_main_quit(event_loop);
 		return FALSE;
 	}
@@ -1127,7 +1132,7 @@ static int run_proxy(int fd, int dev, bdaddr_t *bdaddr)
 	while (!__io_canceled) {
 		p[0].revents = 0;
 		p[1].revents = 0;
-		err = poll(p, 2, 100);
+		err = poll(p, 2, 500);
 		if (err < 0)
 			break;
 		if (!err)
@@ -1137,7 +1142,7 @@ static int run_proxy(int fd, int dev, bdaddr_t *bdaddr)
 			len = read(fd, buf, sizeof(buf));
 			if (len > 0) {
 				rewrite_bdaddr(buf, len, bdaddr);
-				write(dd, buf, len);
+				err = write(dd, buf, len);
 			}
 		}
 
@@ -1145,7 +1150,7 @@ static int run_proxy(int fd, int dev, bdaddr_t *bdaddr)
 			len = read(dd, buf, sizeof(buf));
 			if (len > 0) {
 				rewrite_bdaddr(buf, len, bdaddr);
-				write(fd, buf, len);
+				err = write(fd, buf, len);
 			}
 		}
 	}
@@ -1179,19 +1184,16 @@ static struct option main_options[] = {
 	{ 0 }
 };
 
-int main(int argc, char *argv[], char *env[])
+int main(int argc, char *argv[])
 {
 	struct sigaction sa;
 	GIOChannel *dev_io;
 	char *device = NULL, *snoop = NULL;
 	bdaddr_t bdaddr;
-	int fd, dd, opt, daemon, dofork, dev = -1;
+	int fd, dd, opt, detach = 1, dev = -1;
 
 	bacpy(&bdaddr, BDADDR_ANY);
 
-	/* Configure default settings */
-	daemon = 1; dofork = 1;
-
 	while ((opt=getopt_long(argc, argv, "d:b:s:nh", main_options, NULL)) != EOF) {
 		switch(opt) {
 		case 'd':
@@ -1207,7 +1209,7 @@ int main(int argc, char *argv[], char *env[])
 			break;
 
 		case 'n':
-			daemon = 0;
+			detach = 0;
 			break;
 
 		case 'h':
@@ -1237,18 +1239,11 @@ int main(int argc, char *argv[], char *env[])
 			exit(1);
 	}
 
-	if (daemon) {
-		if (dofork && fork())
-			exit(0);
-
-		/* Direct stdin,stdout,stderr to '/dev/null' */
-		fd = open("/dev/null", O_RDWR);
-		dup2(fd, 0); dup2(fd, 1); dup2(fd, 2);
-		close(fd);
-
-		setsid();
-
-		chdir("/");
+	if (detach) {
+		if (daemon(0, 0)) {
+			perror("Can't start daemon");
+			exit(1);
+		}
 	}
 
 	/* Start logging to syslog and stderr */
