@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
+#include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <getopt.h>
@@ -57,7 +58,8 @@ enum {
 	CRECV,
 	LSEND,
 	SENDDUMP,
-	LSENDDUMP
+	LSENDDUMP,
+	INFOREQ
 };
 
 static unsigned char *buf;
@@ -76,6 +78,14 @@ static unsigned short psm = 10;
 /* Default number of frames to send (-1 = infinite) */
 static int num_frames = -1;
 
+/* Default number of consecutive frames before the delay */
+static int count = 1;
+
+/* Default delay after sending count number of frames */
+static unsigned long delay = 0;
+
+static char *filename = NULL;
+
 static int flowctl = 0;
 static int master = 0;
 static int auth = 0;
@@ -558,15 +568,27 @@ static void recv_mode(int sk)
 	}
 }
 
-static void send_mode(int sk)
+static void do_send(int sk)
 {
 	uint32_t seq;
-	int i, len;
+	int i, fd, len;
 
 	syslog(LOG_INFO, "Sending ...");
 
-	for (i = 6; i < data_size; i++)
-		buf[i] = 0x7f;
+	if (filename) {
+		fd = open(filename, O_RDONLY);
+		if (fd < 0) {
+			syslog(LOG_ERR, "Open failed: %s (%d)",
+							strerror(errno), errno);
+			exit(1);
+		}
+		len = read(fd, buf, data_size);
+		send(sk, buf, len, 0);
+		return;
+	} else {
+		for (i = 6; i < data_size; i++)
+			buf[i] = 0x7f;
+	}
 
 	seq = 0;
 	while ((num_frames == -1) || (num_frames-- > 0)) {
@@ -576,10 +598,19 @@ static void send_mode(int sk)
 
 		len = send(sk, buf, data_size, 0);
 		if (len < 0 || len != data_size) {
-			syslog(LOG_ERR, "Send failed: %s (%d)", strerror(errno), errno);
+			syslog(LOG_ERR, "Send failed: %s (%d)",
+							strerror(errno), errno);
 			exit(1);
 		}
+
+		if (num_frames && delay && count && !(seq % count))
+			usleep(delay);
 	}
+}
+
+static void send_mode(int sk)
+{
+	do_send(sk);
 
 	syslog(LOG_INFO, "Closing channel ...");
 	if (shutdown(sk, SHUT_RDWR) < 0)
@@ -590,25 +621,7 @@ static void send_mode(int sk)
 
 static void senddump_mode(int sk)
 {
-	uint32_t seq;
-	int i;
-
-	syslog(LOG_INFO, "Sending ...");
-
-	for (i = 6; i < data_size; i++)
-		buf[i] = 0x7f;
-
-	seq = 0;
-	while ((num_frames == -1) || (num_frames-- > 0)) {
-		*(uint32_t *) buf = htobl(seq);
-		*(uint16_t *) (buf + 4) = htobs(data_size);
-		seq++;
-
-		if (send(sk, buf, data_size, 0) <= 0) {
-			syslog(LOG_ERR, "Send failed: %s (%d)", strerror(errno), errno);
-			exit(1);
-		}
-	}
+	do_send(sk);
 
 	dump_mode(sk);
 }
@@ -634,7 +647,7 @@ static void connect_mode(char *svr)
 
 	while (1) {
 		p.revents = 0;
-		if (poll(&p, 1, 100))
+		if (poll(&p, 1, 500))
 			break;
 	}
 
@@ -643,23 +656,126 @@ static void connect_mode(char *svr)
 	close(sk);
 }
 
-static void multi_connect_mode(char *svr)
+static void multi_connect_mode(int argc, char *argv[])
 {
-	while (1) {
-		int i, s;
-		for (i = 0; i < 10; i++) {
-			if (fork()) continue;
+	int i, n, sk;
 
-			/* Child */
-			s = do_connect(svr);
-			usleep(500);
-			close(s);
-			exit(0);
+	while (1) {
+		for (n = 0; n < argc; n++) {
+			for (i = 0; i < count; i++) {
+				if (fork())
+					continue;
+
+				/* Child */
+				sk = do_connect(argv[n]);
+				usleep(500);
+				close(sk);
+				exit(0);
+			}
 		}
-		sleep(2);
+		sleep(4);
 	}
 }
 
+static void info_request(char *svr)
+{
+	unsigned char buf[48];
+	l2cap_cmd_hdr *cmd = (l2cap_cmd_hdr *) buf;
+	l2cap_info_req *req = (l2cap_info_req *) (buf + L2CAP_CMD_HDR_SIZE);
+	l2cap_info_rsp *rsp = (l2cap_info_rsp *) (buf + L2CAP_CMD_HDR_SIZE);
+	uint16_t mtu;
+	uint32_t mask;
+	struct sockaddr_l2 addr;
+	int sk, err;
+
+	sk = socket(PF_BLUETOOTH, SOCK_RAW, BTPROTO_L2CAP);
+	if (sk < 0) {
+		perror("Can't create socket");
+		return;
+	}
+
+	memset(&addr, 0, sizeof(addr));
+	addr.l2_family = AF_BLUETOOTH;
+	bacpy(&addr.l2_bdaddr, &bdaddr);
+
+	if (bind(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
+		perror("Can't bind socket");
+		goto failed;
+	}
+
+	memset(&addr, 0, sizeof(addr));
+	addr.l2_family = AF_BLUETOOTH;
+	str2ba(svr, &addr.l2_bdaddr);
+
+	if (connect(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0 ) {
+		perror("Can't connect socket");
+		goto failed;
+	}
+
+	memset(buf, 0, sizeof(buf));
+	cmd->code  = L2CAP_INFO_REQ;
+	cmd->ident = 42;
+	cmd->len   = htobs(2);
+	req->type  = htobs(0x0001);
+
+	if (send(sk, buf, L2CAP_CMD_HDR_SIZE + L2CAP_INFO_REQ_SIZE, 0) < 0) {
+		perror("Can't send info request");
+		goto failed;
+	}
+
+	err = recv(sk, buf, L2CAP_CMD_HDR_SIZE + L2CAP_INFO_RSP_SIZE + 2, 0);
+	if (err < 0) {
+		perror("Can't receive info response");
+		goto failed;
+	}
+
+	switch (btohs(rsp->result)) {
+	case 0x0000:
+		mtu = btohs(bt_get_unaligned((uint16_t *) rsp->data));
+		printf("Connectionless MTU size is %d\n", mtu);
+		break;
+	case 0x0001:
+		printf("Connectionless MTU is not supported\n");
+		break;
+	}
+
+	memset(buf, 0, sizeof(buf));
+	cmd->code  = L2CAP_INFO_REQ;
+	cmd->ident = 42;
+	cmd->len   = htobs(2);
+	req->type  = htobs(0x0002);
+
+	if (send(sk, buf, L2CAP_CMD_HDR_SIZE + L2CAP_INFO_REQ_SIZE, 0) < 0) {
+		perror("Can't send info request");
+		goto failed;
+	}
+
+	err = recv(sk, buf, L2CAP_CMD_HDR_SIZE + L2CAP_INFO_RSP_SIZE + 4, 0);
+	if (err < 0) {
+		perror("Can't receive info response");
+		goto failed;
+	}
+
+	switch (btohs(rsp->result)) {
+	case 0x0000:
+		mask = btohl(bt_get_unaligned((uint32_t *) rsp->data));
+		printf("Extended feature mask is 0x%04x\n", mask);
+		if (mask & 0x01)
+			printf("  Flow control mode\n");
+		if (mask & 0x02)
+			printf("  Retransmission mode\n");
+		if (mask & 0x04)
+			printf("  Bi-directional QoS\n");
+		break;
+	case 0x0001:
+		printf("Extended feature mask is not supported\n");
+		break;
+	}
+
+failed:
+	close(sk);
+}
+
 static void usage(void)
 {
 	printf("l2test - L2CAP testing\n"
@@ -675,15 +791,19 @@ static void usage(void)
 		"\t-n connect and be silent\n"
 		"\t-y connect, then send, then dump incoming data\n"
 		"\t-c connect, disconnect, connect, ...\n"
-		"\t-m multiple connects\n");
+		"\t-m multiple connects\n"
+		"\t-z information request\n");
 
 	printf("Options:\n"
 		"\t[-b bytes] [-i device] [-P psm]\n"
 		"\t[-I imtu] [-O omtu]\n"
-		"\t[-N num] send num frames (default = infinite)\n"
 		"\t[-L seconds] enable SO_LINGER\n"
+		"\t[-B filename] use data packets from file\n"
+		"\t[-N num] send num frames (default = infinite)\n"
+		"\t[-C num] send num frames before delay (default = 1)\n"
+		"\t[-D milliseconds] delay after sending num frames (default = 0)\n"
 		"\t[-R] reliable mode\n"
-		"\t[-D] use connectionless channel (datagram)\n"
+		"\t[-G] use connectionless channel (datagram)\n"
 		"\t[-F] enable flow control\n"
 		"\t[-A] request authentication\n"
 		"\t[-E] request encryption\n"
@@ -691,14 +811,14 @@ static void usage(void)
 		"\t[-M] become master\n");
 }
 
-int main(int argc ,char *argv[])
+int main(int argc, char *argv[])
 {
 	struct sigaction sa;
 	int opt, sk, mode = RECV, need_addr = 0;
 
 	bacpy(&bdaddr, BDADDR_ANY);
 
-	while ((opt=getopt(argc,argv,"rdscuwmnxyb:i:P:I:O:N:L:RDFAESM")) != EOF) {
+	while ((opt=getopt(argc,argv,"rdscuwmnxyzb:i:P:I:O:B:N:L:C:D:RGFAESM")) != EOF) {
 		switch(opt) {
 		case 'r':
 			mode = RECV;
@@ -737,10 +857,6 @@ int main(int argc ,char *argv[])
 			need_addr = 1;
 			break;
 
-		case 'b':
-			data_size = atoi(optarg);
-			break;
-
 		case 'x':
 			mode = LSENDDUMP;
 			break;
@@ -749,6 +865,15 @@ int main(int argc ,char *argv[])
 			mode = SENDDUMP;
 			break;
 
+		case 'z':
+			mode = INFOREQ;
+			need_addr = 1;
+			break;
+
+		case 'b':
+			data_size = atoi(optarg);
+			break;
+
 		case 'i':
 			if (!strncasecmp(optarg, "hci", 3))
 				hci_devba(atoi(optarg + 3), &bdaddr);
@@ -772,6 +897,22 @@ int main(int argc ,char *argv[])
 			linger = atoi(optarg);
 			break;
 
+		case 'B':
+			filename = strdup(optarg);
+			break;
+
+		case 'N':
+			num_frames = atoi(optarg);
+			break;
+
+		case 'C':
+			count = atoi(optarg);
+			break;
+
+		case 'D':
+			delay = atoi(optarg) * 1000;
+			break;
+
 		case 'R':
 			reliable = 1;
 			break;
@@ -796,14 +937,10 @@ int main(int argc ,char *argv[])
 			secure = 1;
 			break;
 
-		case 'D':
+		case 'G':
 			socktype = SOCK_DGRAM;
 			break;
 
-		case 'N':
-			num_frames = atoi(optarg);
-			break;
-
 		default:
 			usage();
 			exit(1);
@@ -867,7 +1004,7 @@ int main(int argc ,char *argv[])
 			break;
 
 		case MULTY:
-			multi_connect_mode(argv[optind]);
+			multi_connect_mode(argc - optind, argv + optind);
 			break;
 
 		case CONNECT:
@@ -884,6 +1021,10 @@ int main(int argc ,char *argv[])
 		case LSENDDUMP:
 			do_listen(senddump_mode);
 			break;
+
+		case INFOREQ:
+			info_request(argv[optind]);
+			exit(0);
 	}
 
 	syslog(LOG_INFO, "Exit");
