@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
+#include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <getopt.h>
@@ -60,10 +61,18 @@ static unsigned char *buf;
 static long data_size = 127;
 static long num_frames = -1;
 
+/* Default number of consecutive frames before the delay */
+static int count = 1;
+
+/* Default delay after sending count number of frames */
+static unsigned long delay = 0;
+
 /* Default addr and channel */
 static bdaddr_t bdaddr;
 static uint8_t channel = 10;
 
+static char *filename = NULL;
+
 static int master = 0;
 static int auth = 0;
 static int encrypt = 0;
@@ -323,15 +332,27 @@ static void recv_mode(int sk)
 	}
 }
 
-static void send_mode(int sk)
+static void do_send(int sk)
 {
 	uint32_t seq;
-	int i;
+	int i, fd, len;
 
 	syslog(LOG_INFO,"Sending ...");
 
-	for (i = 6; i < data_size; i++)
-		buf[i] = 0x7f;
+	if (filename) {
+		fd = open(filename, O_RDONLY);
+		if (fd < 0) {
+			syslog(LOG_ERR, "Open failed: %s (%d)",
+							strerror(errno), errno);
+			exit(1);
+		}
+		len = read(fd, buf, data_size);
+		send(sk, buf, len, 0);
+		return;
+	} else {
+		for (i = 6; i < data_size; i++)
+			buf[i] = 0x7f;
+	}
 
 	seq = 0;
 	while ((num_frames == -1) || (num_frames-- > 0)) {
@@ -344,7 +365,15 @@ static void send_mode(int sk)
 							strerror(errno), errno);
 			exit(1);
 		}
+
+		if (num_frames && delay && count && !(seq % count))
+			usleep(delay);
 	}
+}
+
+static void send_mode(int sk)
+{
+	do_send(sk);
 
 	syslog(LOG_INFO, "Closing channel ...");
 	if (shutdown(sk, SHUT_RDWR) < 0)
@@ -361,22 +390,24 @@ static void reconnect_mode(char *svr)
 	}
 }
 
-static void multi_connect_mode(char *svr)
+static void multi_connect_mode(int argc, char *argv[])
 {
+	int i, n, sk;
+
 	while (1) {
-		int i, sk;
+		for (n = 0; n < argc; n++) {
+			for (i = 0; i < count; i++) {
+				if (fork())
+					continue;
 
-		for (i = 0; i < 10; i++) {
-			if (fork()) continue;
-
-			/* Child */
-			sk = do_connect(svr);
-			usleep(500);
-			close(sk);
-			exit(0);
+				/* Child */
+				sk = do_connect(argv[n]);
+				usleep(500);
+				close(sk);
+				exit(0);
+			}
 		}
-
-		sleep(2);
+		sleep(4);
 	}
 }
 
@@ -398,21 +429,24 @@ static void usage(void)
 	printf("Options:\n"
 		"\t[-b bytes] [-i device] [-P channel]\n"
 		"\t[-L seconds] enabled SO_LINGER option\n"
+		"\t[-B filename] use data packets from file\n"
 		"\t[-N num] number of frames to send\n"
+		"\t[-C num] send num frames before delay (default = 1)\n"
+		"\t[-D milliseconds] delay after sending num frames (default = 0)\n"
 		"\t[-A] request authentication\n"
 		"\t[-E] request encryption\n"
 		"\t[-S] secure connection\n"
 		"\t[-M] become master\n");
 }
 
-int main(int argc ,char *argv[])
+int main(int argc, char *argv[])
 {
 	struct sigaction sa;
 	int opt, sk, mode = RECV, need_addr = 0;
 
 	bacpy(&bdaddr, BDADDR_ANY);
 
-	while ((opt=getopt(argc,argv,"rdscuwmnb:i:P:N:MAESL:")) != EOF) {
+	while ((opt=getopt(argc,argv,"rdscuwmnb:i:P:B:N:MAESL:C:D:")) != EOF) {
 		switch (opt) {
 		case 'r':
 			mode = RECV;
@@ -486,10 +520,22 @@ int main(int argc ,char *argv[])
 			linger = atoi(optarg);
 			break;
 
+		case 'B':
+			filename = strdup(optarg);
+			break;
+
 		case 'N':
 			num_frames = atoi(optarg);
 			break;
 
+		case 'C':
+			count = atoi(optarg);
+			break;
+
+		case 'D':
+			delay = atoi(optarg) * 1000;
+			break;
+
 		default:
 			usage();
 			exit(1);
@@ -545,7 +591,7 @@ int main(int argc ,char *argv[])
 			break;
 
 		case MULTY:
-			multi_connect_mode(argv[optind]);
+			multi_connect_mode(argc - optind, argv + optind);
 			break;
 
 		case CONNECT:
