@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -199,9 +199,9 @@ static char *memorytostr(uint16_t type)
 	}
 }
 
-#define OPT_RANGE(range) \
-		if (argc < (range)) { errno = EINVAL; return -1; } \
-		if (argc > (range)) { errno = E2BIG; return -1; }
+#define OPT_RANGE(min, max) \
+		if (argc < (min)) { errno = EINVAL; return -1; } \
+		if (argc > (max)) { errno = E2BIG; return -1; }
 
 static struct option help_options[] = {
 	{ "help",	0, 0, 'h' },
@@ -227,7 +227,7 @@ static int opt_help(int argc, char *argv[], int *help)
 #define OPT_HELP(range, help) \
 		opt_help(argc, argv, (help)); \
 		argc -= optind; argv += optind; optind = 0; \
-		OPT_RANGE((range))
+		OPT_RANGE((range), (range))
 
 static int cmd_builddef(int transport, int argc, char *argv[])
 {
@@ -429,6 +429,60 @@ static int cmd_enabletx(int transport, int argc, char *argv[])
 	return transport_write(transport, CSR_VARID_ENABLE_TX, NULL, 0);
 }
 
+static int cmd_singlechan(int transport, int argc, char *argv[])
+{
+	uint8_t array[8];
+	uint16_t channel;
+
+	OPT_HELP(1, NULL);
+
+	channel = atoi(argv[0]);
+
+	if (channel > 2401 && channel < 2481)
+		channel -= 2402;
+
+	if (channel > 78) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	memset(array, 0, sizeof(array));
+	array[0] = channel & 0xff;
+	array[1] = channel >> 8;
+
+	return transport_write(transport, CSR_VARID_SINGLE_CHAN, array, 8);
+}
+
+static int cmd_hoppingon(int transport, int argc, char *argv[])
+{
+	return transport_write(transport, CSR_VARID_HOPPING_ON, NULL, 0);
+}
+
+static int cmd_rttxdata1(int transport, int argc, char *argv[])
+{
+	uint8_t array[8];
+	uint16_t freq, level;
+
+	OPT_HELP(2, NULL);
+
+	freq = atoi(argv[0]);
+
+	if (!strncasecmp(argv[1], "0x", 2))
+		level = strtol(argv[1], NULL, 16);
+	else
+		level = atoi(argv[1]);
+
+	memset(array, 0, sizeof(array));
+	array[0] = 0x04;
+	array[1] = 0x00;
+	array[2] = freq & 0xff;
+	array[3] = freq >> 8;
+	array[4] = level & 0xff;
+	array[5] = level >> 8;
+
+	return transport_write(transport, CSR_VARID_RADIOTEST, array, 8);
+}
+
 static int cmd_memtypes(int transport, int argc, char *argv[])
 {
 	uint8_t array[8];
@@ -510,10 +564,10 @@ static int opt_pskey(int argc, char *argv[], uint16_t *stores, int *reset, int *
 	return optind;
 }
 
-#define OPT_PSKEY(range, stores, reset, help) \
+#define OPT_PSKEY(min, max, stores, reset, help) \
 		opt_pskey(argc, argv, (stores), (reset), (help)); \
 		argc -= optind; argv += optind; optind = 0; \
-		OPT_RANGE((range))
+		OPT_RANGE((min), (max))
 
 static int cmd_psget(int transport, int argc, char *argv[])
 {
@@ -524,7 +578,7 @@ static int cmd_psget(int transport, int argc, char *argv[])
 
 	memset(array, 0, sizeof(array));
 
-	OPT_PSKEY(1, &stores, &reset, NULL);
+	OPT_PSKEY(1, 1, &stores, &reset, NULL);
 
 	if (strncasecmp(argv[0], "0x", 2)) {
 		pskey = atoi(argv[0]);
@@ -599,7 +653,7 @@ static int cmd_psset(int transport, int argc, char *argv[])
 
 	memset(array, 0, sizeof(array));
 
-	OPT_PSKEY(2, &stores, &reset, NULL);
+	OPT_PSKEY(2, 81, &stores, &reset, NULL);
 
 	if (strncasecmp(argv[0], "0x", 2)) {
 		pskey = atoi(argv[0]);
@@ -702,7 +756,7 @@ static int cmd_psclr(int transport, int argc, char *argv[])
 	uint16_t pskey, stores = CSR_STORES_PSRAM;
 	int i, err, reset = 0;
 
-	OPT_PSKEY(1, &stores, &reset, NULL);
+	OPT_PSKEY(1, 1, &stores, &reset, NULL);
 
 	if (strncasecmp(argv[0], "0x", 2)) {
 		pskey = atoi(argv[0]);
@@ -739,7 +793,7 @@ static int cmd_pslist(int transport, int argc, char *argv[])
 	uint16_t pskey = 0x0000, length, stores = CSR_STORES_DEFAULT;
 	int err, reset = 0;
 
-	OPT_PSKEY(0, &stores, &reset, NULL);
+	OPT_PSKEY(0, 0, &stores, &reset, NULL);
 
 	while (1) {
 		memset(array, 0, sizeof(array));
@@ -785,7 +839,7 @@ static int cmd_psread(int transport, int argc, char *argv[])
 	char *str, val[7];
 	int i, err, reset = 0;
 
-	OPT_PSKEY(0, &stores, &reset, NULL);
+	OPT_PSKEY(0, 0, &stores, &reset, NULL);
 
 	while (1) {
 		memset(array, 0, sizeof(array));
@@ -854,7 +908,7 @@ static int cmd_psload(int transport, int argc, char *argv[])
 	char *str, val[7];
 	int err, reset = 0;
 
-	OPT_PSKEY(1, &stores, &reset, NULL);
+	OPT_PSKEY(1, 1, &stores, &reset, NULL);
 
 	psr_read(argv[0]);
 
@@ -932,6 +986,9 @@ static struct {
 	{ "warmreset", cmd_warmreset, "",              "Perform warm reset"             },
 	{ "disabletx", cmd_disabletx, "",              "Disable TX on the device"       },
 	{ "enabletx",  cmd_enabletx,  "",              "Enable TX on the device"        },
+	{ "singlechan",cmd_singlechan,"<channel>",     "Lock radio on specific channel" },
+	{ "hoppingon", cmd_hoppingon, "",              "Revert to channel hopping"      },
+	{ "rttxdata1", cmd_rttxdata1, "<freq> <level>","TXData1 radio test"             },
 	{ "memtypes",  cmd_memtypes,  NULL,            "Get memory types"               },
 	{ "psget",     cmd_psget,     "<key>",         "Get value for PS key"           },
 	{ "psset",     cmd_psset,     "<key> <value>", "Set value for PS key"           },
