@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -320,9 +320,23 @@ static struct {
 	{ 2655, "Unified 21c"	},
 	{ 2656, "Unified 21c"	},
 	{ 2658, "Unified 21c"	},
+	{ 3057, "Unified 21d"	},
+	{ 3058, "Unified 21d"	},	
+	{ 3059, "Unified 21d"	},
+	{ 3060, "Unified 21d"	},
+	{ 3062, "Unified 21d"	},
+	{ 3063, "Unified 21d"	},
+	{ 3064, "Unified 21d"	},
+	{ 3164, "Unified 21e"	},
+	{ 3413, "Unified 21f"	},
+	{ 3414, "Unified 21f"	},
+	{ 3415, "Unified 21f"	},
+	{ 3424, "Unified 21f"	},
+	{ 3454, "Unified 21f"	},
 	{ 2526, "Marcel 1 (2005-09-26)"	},
 	{ 2543, "Marcel 2 (2005-09-28)"	},
 	{ 2622, "Marcel 3 (2005-10-27)" },
+	{ 3326, "Marcel 4 (2006-06-16)" },
 	{  195, "Sniff 1 (2001-11-27)"	},
 	{  220, "Sniff 2 (2002-01-03)"	},
 	{  269, "Sniff 3 (2002-02-22)"	},
@@ -387,6 +401,12 @@ static struct {
 	{ 2388, "Sniff 46 (2005-08-17)" },
 	{ 2389, "Sniff 46 (2005-08-17)" },
 	{ 2390, "Sniff 46 (2005-08-17)" },
+	{ 2869, "Sniff 47 (2006-02-15)" },
+	{ 2870, "Sniff 47 (2006-02-15)" },
+	{ 2871, "Sniff 47 (2006-02-15)" },
+	{ 3214, "Sniff 48 (2006-05-16)" },
+	{ 3215, "Sniff 48 (2006-05-16)" },
+	{ 3216, "Sniff 48 (2006-05-16)" },
 	{    0, }
 };
 
