@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,8 @@
 #include <termios.h>
 #include <time.h>
 #include <sys/time.h>
+#include <sys/poll.h>
+#include <sys/param.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 
@@ -70,6 +72,23 @@ struct uart_t {
 
 #define FLOW_CTL	0x0001
 
+static volatile sig_atomic_t __io_canceled = 0;
+
+static void sig_hup(int sig)
+{
+}
+
+static void sig_term(int sig)
+{
+	__io_canceled = 1;
+}
+
+static void sig_alarm(int sig)
+{
+	fprintf(stderr, "Initialization timed out.\n");
+	exit(1);
+}
+
 static int uart_speed(int s)
 {
 	switch (s) {
@@ -110,12 +129,6 @@ static int set_speed(int fd, struct termios *ti, int speed)
 	return tcsetattr(fd, TCSANOW, ti);
 }
 
-static void sig_alarm(int sig)
-{
-	fprintf(stderr, "Initialization timed out.\n");
-	exit(1);
-}
-
 /* 
  * Read an HCI event from the given file descriptor.
  */
@@ -158,6 +171,7 @@ static int read_hci_event(int fd, unsigned char* buf, int size)
 			return -1;
 		count += r;
 	}
+
 	return count;
 }
 
@@ -201,6 +215,7 @@ static int ericsson(int fd, struct uart_t *u, struct termios *ti)
 		perror("Failed to write init command");
 		return -1;
 	}
+
 	nanosleep(&tm, NULL);
 	return 0;
 }
@@ -237,6 +252,7 @@ static int digi(int fd, struct uart_t *u, struct termios *ti)
 		perror("Failed to write init command");
 		return -1;
 	}
+
 	nanosleep(&tm, NULL);
 	return 0;
 }
@@ -289,7 +305,7 @@ static int texas(int fd, struct uart_t *u, struct termios *ti)
 
 	/* Print LMP subversion */
 	fprintf(stderr, "Texas module LMP sub-version : 0x%02x%02x\n", resp[14] & 0xFF, resp[13] & 0xFF);
-	
+
 	nanosleep(&tm, NULL);
 	return 0;
 }
@@ -297,7 +313,7 @@ static int texas(int fd, struct uart_t *u, struct termios *ti)
 static int read_check(int fd, void *buf, int count)
 {
 	int res;
-	
+
 	do {
 		res = read(fd, buf, count);
 		if (res != -1) {
@@ -305,10 +321,10 @@ static int read_check(int fd, void *buf, int count)
 			count -= res;
 		}
 	} while (count && (errno == 0 || errno == EINTR));
-	
+
 	if (count)
 		return -1;
-	
+
 	return 0;
 }
 
@@ -321,13 +337,15 @@ static void bcsp_tshy_sig_alarm(int sig)
 {
 	static int retries=0;
 	unsigned char bcsp_sync_pkt[10] = {0xc0,0x00,0x41,0x00,0xbe,0xda,0xdc,0xed,0xed,0xc0};
-	
+	int len;
+
 	if (retries < 10) {
 		retries++;
-		write(serial_fd, &bcsp_sync_pkt, 10);
+		len = write(serial_fd, &bcsp_sync_pkt, 10);
 		alarm(1);
 		return;
 	}
+
 	tcflush(serial_fd, TCIOFLUSH);
 	fprintf(stderr, "BCSP initialization timed out\n");
 	exit(1);
@@ -337,12 +355,15 @@ static void bcsp_tconf_sig_alarm(int sig)
 {
 	static int retries=0;
 	unsigned char bcsp_conf_pkt[10] = {0xc0,0x00,0x41,0x00,0xbe,0xad,0xef,0xac,0xed,0xc0};
+	int len;
+
 	if (retries < 10){
 		retries++;
-		write(serial_fd, &bcsp_conf_pkt, 10);
+		len = write(serial_fd, &bcsp_conf_pkt, 10);
 		alarm(1);
 		return;
 	}
+
 	tcflush(serial_fd, TCIOFLUSH);
 	fprintf(stderr, "BCSP initialization timed out\n");
 	exit(1);
@@ -358,6 +379,7 @@ static int bcsp(int fd, struct uart_t *u, struct termios *ti)
 		bcspconf[4]     = {0xad,0xef,0xac,0xed},
 		bcspconfresp[4] = {0xde,0xad,0xd0,0xd0};
 	struct sigaction sa;
+	int len;
 
 	if (set_speed(fd, ti, u->speed) < 0) {
 		perror("Can't set default baud rate");
@@ -390,20 +412,19 @@ static int bcsp(int fd, struct uart_t *u, struct termios *ti)
 				return -1;
 			}
 		} while (byte != 0xC0);
-		
+
 		do {
 			if ( read_check(fd, &bcsph[0], 1) == -1){
 				perror("Failed to read");
 				return -1;
 			}
-		  
 		} while (bcsph[0] == 0xC0);
-		
+
 		if ( read_check(fd, &bcsph[1], 3) == -1){
 			perror("Failed to read");
 			return -1;
 		}
-		
+
 		if (((bcsph[0] + bcsph[1] + bcsph[2]) & 0xFF) != (unsigned char)~bcsph[3])
 			continue;
 		if (bcsph[1] != 0x41 || bcsph[2] != 0x00)
@@ -415,7 +436,7 @@ static int bcsp(int fd, struct uart_t *u, struct termios *ti)
 		}
 
 		if (!memcmp(bcspp, bcspsync, 4)) {
-			write(fd, &bcsp_sync_resp_pkt,10);
+			len = write(fd, &bcsp_sync_resp_pkt,10);
 		} else if (!memcmp(bcspp, bcspsyncresp, 4))
 			break;
 	}
@@ -437,8 +458,8 @@ static int bcsp(int fd, struct uart_t *u, struct termios *ti)
 
 		do {
 			if (read_check(fd, &bcsph[0], 1) == -1){
-			      perror("Failed to read");
-			      return -1;
+				perror("Failed to read");
+				return -1;
 			}
 		} while (bcsph[0] == 0xC0);
 
@@ -446,7 +467,7 @@ static int bcsp(int fd, struct uart_t *u, struct termios *ti)
 			perror("Failed to read");
 			return -1;
 		}
-		
+
 		if (((bcsph[0] + bcsph[1] + bcsph[2]) & 0xFF) != (unsigned char)~bcsph[3])
 			continue;
 
@@ -459,9 +480,9 @@ static int bcsp(int fd, struct uart_t *u, struct termios *ti)
 		}
 
 		if (!memcmp(bcspp, bcspsync, 4))
-			write(fd, &bcsp_sync_resp_pkt, 10);
+			len = write(fd, &bcsp_sync_resp_pkt, 10);
 		else if (!memcmp(bcspp, bcspconf, 4))
-			write(fd, &bcsp_conf_resp_pkt, 10);
+			len = write(fd, &bcsp_conf_resp_pkt, 10);
 		else if (!memcmp(bcspp, bcspconfresp,  4))
 			break;
 	}
@@ -553,7 +574,7 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 	/* Display that to user */
 	fprintf(stderr, "CSR build ID 0x%02X-0x%02X\n", 
 		resp[15] & 0xFF, resp[14] & 0xFF);
-	
+
 	/* Try to read the current speed of the CSR chip */
 	clen = 5 + (5 + 4)*2;
 	/* -- HCI header */
@@ -638,6 +659,7 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 		perror("Failed to write init command (SET_UART_SPEED)");
 		return -1;
 	}
+
 	nanosleep(&tm, NULL);
 	return 0;
 }
@@ -715,7 +737,7 @@ static int swave(int fd, struct uart_t *u, struct termios *ti)
 		//  01	flow on
 		//  01 	Hci Transport type = Uart
 		//  xx	Baud rate set (see above)
-	} else {	
+	} else {
 		// ups, got error.
 		return -1;
 	}
@@ -735,7 +757,7 @@ static int swave(int fd, struct uart_t *u, struct termios *ti)
 	}
 
 	nanosleep(&tm, NULL);
-			
+
 	// now the uart baud rate on the silicon wave module is set and effective.
 	// change our own baud rate as well. Then there is a reset event comming in
  	// on the *new* baud rate. This is *undocumented*! The packet looks like this:
@@ -798,10 +820,136 @@ static int st(int fd, struct uart_t *u, struct termios *ti)
 		perror("Failed to write init command");
 		return -1;
 	}
+
 	nanosleep(&tm, NULL);
 	return 0;
 }
 
+extern int stlc2500_init(int fd);
+
+static int stlc2500(int fd, struct uart_t *u, struct termios *ti)
+{
+	return stlc2500_init(fd);
+}
+
+/*
+ * Broadcom specific initialization
+ * Extracted from Jungo openrg
+ */
+static int bcm2035(int fd, struct uart_t *u, struct termios *ti)
+{
+	int n;
+	unsigned char cmd[30], resp[30];
+
+	/* Reset the BT Chip */
+	memset(cmd, 0, sizeof(cmd));
+	memset(resp, 0, sizeof(resp));
+	cmd[0] = HCI_COMMAND_PKT;
+	cmd[1] = 0x03;
+	cmd[2] = 0x0c;
+	cmd[3] = 0x00;
+
+	/* Send command */
+	if (write(fd, cmd, 4) != 4) {
+		fprintf(stderr, "Failed to write reset command\n");
+		return -1;
+	}
+
+	/* Read reply */
+	if ((n = read_hci_event(fd, resp, 4)) < 0) {
+		fprintf(stderr, "Failed to reset chip\n");
+		return -1;
+	}
+
+	/* Read the local version info */
+	memset(cmd, 0, sizeof(cmd));
+	memset(resp, 0, sizeof(resp));
+	cmd[0] = HCI_COMMAND_PKT;
+	cmd[1] = 0x01;
+	cmd[2] = 0x10;
+	cmd[3] = 0x00;
+
+	/* Send command */
+	if (write(fd, cmd, 4) != 4) {
+		fprintf(stderr, "Failed to write \"read local version\" "
+			"command\n");
+		return -1;
+	}
+
+	/* Read reply */
+	if ((n = read_hci_event(fd, resp, 4)) < 0) {
+		fprintf(stderr, "Failed to read local version\n");
+		return -1;
+	}
+
+	/* Read the local supported commands info */
+	memset(cmd, 0, sizeof(cmd));
+	memset(resp, 0, sizeof(resp));
+	cmd[0] = HCI_COMMAND_PKT;
+	cmd[1] = 0x02;
+	cmd[2] = 0x10;
+	cmd[3] = 0x00;
+
+	/* Send command */
+	if (write(fd, cmd, 4) != 4) {
+		fprintf(stderr, "Failed to write \"read local supported "
+			"commands\" command\n");
+		return -1;
+	}
+
+	/* Read reply */
+	if ((n = read_hci_event(fd, resp, 4)) < 0) {
+		fprintf(stderr, "Failed to read local supported commands\n");
+		return -1;
+	}
+
+	/* Set the baud rate */
+	memset(cmd, 0, sizeof(cmd));
+	memset(resp, 0, sizeof(resp));
+	cmd[0] = HCI_COMMAND_PKT;
+	cmd[1] = 0x18;
+	cmd[2] = 0xfc;
+	cmd[3] = 0x02;
+	switch (u->speed) {
+	case 57600:
+		cmd[4] = 0x00;
+		cmd[5] = 0xe6;
+		break;
+	case 230400:
+		cmd[4] = 0x22;
+		cmd[5] = 0xfa;
+		break;
+	case 460800:
+		cmd[4] = 0x11;
+		cmd[5] = 0xfd;
+		break;
+	case 921600:
+		cmd[4] = 0x65;
+		cmd[5] = 0xff;
+		break;
+	default:
+		/* Default is 115200 */
+		cmd[4] = 0x00;
+		cmd[5] = 0xf3;
+		break;
+	}
+	fprintf(stderr, "Baud rate parameters: DHBR=0x%2x,DLBR=0x%2x\n",
+		cmd[4], cmd[5]);
+
+	/* Send command */
+	if (write(fd, cmd, 6) != 6) {
+		fprintf(stderr, "Failed to write \"set baud rate\" command\n");
+		return -1;
+	}
+
+	if ((n = read_hci_event(fd, resp, 6)) < 0) {
+		fprintf(stderr, "Failed to set baud rate\n");
+		return -1;
+	}
+
+	return 0;
+}
+
 struct uart_t uart[] = {
 	{ "any",        0x0000, 0x0000, HCI_UART_H4,   115200, 115200, FLOW_CTL, NULL     },
 	{ "ericsson",   0x0000, 0x0000, HCI_UART_H4,   57600,  115200, FLOW_CTL, ericsson },
@@ -825,6 +973,9 @@ struct uart_t uart[] = {
 	/* ST Microelectronics minikits based on STLC2410/STLC2415 */
 	{ "st",         0x0000, 0x0000, HCI_UART_H4,    57600, 115200, FLOW_CTL, st       },
 
+	/* ST Microelectronics minikits based on STLC2500 */
+	{ "stlc2500",   0x0000, 0x0000, HCI_UART_H4,   115200, 115200, FLOW_CTL, stlc2500 },
+
 	/* Sphinx Electronics PICO Card */
 	{ "picocard",   0x025e, 0x1000, HCI_UART_H4,   115200, 115200, FLOW_CTL, NULL     },
 
@@ -855,6 +1006,9 @@ struct uart_t uart[] = {
 	/* Billionton PCBTC1 PCMCIA Card */
 	{ "billionton", 0x0279, 0x950b, HCI_UART_BCSP, 115200, 115200, 0,        bcsp     },
 
+	/* Broadcom BCM2035 */
+	{ "bcm2035",    0x0A5C, 0x2035, HCI_UART_H4,   115200, 115200, 0,        bcm2035  },
+
 	{ NULL, 0 }
 };
 
@@ -882,7 +1036,7 @@ struct uart_t * get_by_type(char *type)
 int init_uart(char *dev, struct uart_t *u, int send_break)
 {
 	struct termios ti;
-	int  fd, i;
+	int fd, i;
 
 	fd = open(dev, O_RDWR | O_NOCTTY);
 	if (fd < 0) {
@@ -957,23 +1111,21 @@ static void usage(void)
 	printf("\thciattach -l\n");
 }
 
-extern int optind, opterr, optopt;
-extern char *optarg;
-
 int main(int argc, char *argv[])
 {
 	struct uart_t *u = NULL;
-	int detach, printpid, opt, i, n;
+	int detach, printpid, opt, i, n, ld;
 	int to = 5; 
 	int init_speed = 0;
 	int send_break = 0;
 	pid_t pid;
 	struct sigaction sa;
+	struct pollfd p;
 	char dev[PATH_MAX];
 
 	detach = 1;
 	printpid = 0;
-	
+
 	while ((opt=getopt(argc, argv, "bnpt:s:l")) != EOF) {
 		switch(opt) {
 		case 'b':
@@ -1017,9 +1169,9 @@ int main(int argc, char *argv[])
 
 	for (n = 0; optind < argc; n++, optind++) {
 		char *opt;
-	
+
 		opt = argv[optind];
-		
+
 		switch(n) {
 		case 0:
 			dev[0] = 0;
@@ -1068,13 +1220,13 @@ int main(int argc, char *argv[])
 		u->init_speed = init_speed;
 
 	memset(&sa, 0, sizeof(sa));
-	sa.sa_flags = SA_NOCLDSTOP;
+	sa.sa_flags   = SA_NOCLDSTOP;
 	sa.sa_handler = sig_alarm;
 	sigaction(SIGALRM, &sa, NULL);
 
 	/* 5 seconds should be enough for initialization */
 	alarm(to);
-	
+
 	n = init_uart(dev, u, send_break);
 	if (n < 0) {
 		perror("Can't initialize device"); 
@@ -1083,16 +1235,46 @@ int main(int argc, char *argv[])
 
 	alarm(0);
 
+	memset(&sa, 0, sizeof(sa));
+	sa.sa_flags   = SA_NOCLDSTOP;
+	sa.sa_handler = SIG_IGN;
+	sigaction(SIGCHLD, &sa, NULL);
+	sigaction(SIGPIPE, &sa, NULL);
+
+	sa.sa_handler = sig_term;
+	sigaction(SIGTERM, &sa, NULL);
+	sigaction(SIGINT,  &sa, NULL);
+
+	sa.sa_handler = sig_hup;
+	sigaction(SIGHUP, &sa, NULL);
+
 	if (detach) {
 		if ((pid = fork())) {
 			if (printpid)
 				printf("%d\n", pid);
 			return 0;
 		}
-		for (i=0; i<20; i++)
-			if (i != n) close(i);
+
+		for (i = 0; i < 20; i++)
+			if (i != n)
+				close(i);
+	}
+
+	p.fd = n;
+	p.events = POLLERR | POLLHUP;
+
+	while (!__io_canceled) {
+		p.revents = 0;
+		if (poll(&p, 1, 500))
+			break;
+	}
+
+	/* Restore TTY line discipline */
+	ld = N_TTY;
+	if (ioctl(n, TIOCSETD, &ld) < 0) {
+		perror("Can't restore line discipline");
+		exit(1);
 	}
 
-	while (1) sleep(999999999);
 	return 0;
 }
