@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <getopt.h>
+#include <sys/param.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 
@@ -666,7 +667,7 @@ static void cmd_class(int ctl, int hdev, char *opt)
 		printf("\tService Classes: ");
 		if (cls[2]) {
 			int first = 1;
-			for (s = 0; s < sizeof(*services); s++)
+			for (s = 0; s < (sizeof(services) / sizeof(*services)); s++)
 				if (cls[2] & (1 << s)) {
 					if (!first)
 						printf(", ");
@@ -676,7 +677,7 @@ static void cmd_class(int ctl, int hdev, char *opt)
 		} else
 			printf("Unspecified");
 		printf("\n\tDevice Class: ");
-		if ((cls[1] & 0x1f) > sizeof(*major_devices))
+		if ((cls[1] & 0x1f) >= sizeof(major_devices) / sizeof(*major_devices))
 			printf("Invalid Device Class!\n");
 		else
 			printf("%s, %s\n", major_devices[cls[1] & 0x1f],
@@ -733,7 +734,7 @@ static int get_link_key(const bdaddr_t *local, const bdaddr_t *peer, uint8_t *ke
 	int i;
 
 	ba2str(local, addr);
-	snprintf(filename, PATH_MAX, "%s/%s/linkkeys", STORAGEDIR, addr);
+	create_name(filename, PATH_MAX, STORAGEDIR, addr, "linkkeys");
 
 	ba2str(peer, addr);
 	str = textfile_get(filename, addr);
@@ -1369,7 +1370,7 @@ static void print_rev_digianswer(int dd)
 
 static void print_rev_broadcom(uint16_t hci_rev, uint16_t lmp_subver)
 {
-	printf("\tFirmware %d.%d / %d\n", hci_rev, lmp_subver >> 8, lmp_subver & 0xff);
+	printf("\tFirmware %d.%d / %d\n", hci_rev & 0xff, lmp_subver >> 8, lmp_subver & 0xff);
 }
 
 static void print_rev_avm(uint16_t hci_rev, uint16_t lmp_subver)
@@ -1532,12 +1533,12 @@ static void usage(void)
 }
 
 static struct option main_options[] = {
-	{"help", 0,0, 'h'},
-	{"all",  0,0, 'a'},
-	{0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ "all",	0, 0, 'a' },
+	{ 0, 0, 0, 0 }
 };
 
-int main(int argc, char **argv, char **env)
+int main(int argc, char *argv[])
 {
 	int opt, ctl, i, cmd=0;
 
